/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_webirc.c: Enables WEBIRC user protocol
 *
 *  Copyright (C) 2008 Sean Rees (eri) <seanrees@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_webirc.c 63 2008-03-02 18:49:37Z srees $
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "client.h"
#include "common.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_user.h"
#include "send.h"
#include "list.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "packet.h"

static void mr_webirc(struct Client *, struct Client *, int, char *[]);

struct Message webirc_msgtab = {
  "WEBIRC", 0, 0, 5, 0, MFLG_SLOW, 0,
  { mr_webirc, m_ignore, m_ignore, m_ignore, m_ignore, m_ignore }
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&webirc_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&webirc_msgtab);
}

const char *_version = "$Revision: 33 $";
#endif

/*
** mr_webirc
**      parv[0] = sender prefix
**      parv[1] = password
**      parv[2] = "cgiirc" (don't care about this param)
**      parv[3] = hostname
**      parv[4] = ip address
*/
static void
mr_webirc(struct Client *client_p, struct Client *source_p,
       int parc, char *parv[])
{
  struct ConfItem *conf;
  struct AccessItem *aconf=NULL;
  const char *password = parv[1];
  const char *hostname = parv[3];
  const char *address = parv[4];

  if (EmptyString(password))
  {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
	       me.name, source_p->name, "WEBIRC");
    return;
  }

  if ((conf = find_exact_name_conf(WEBIRC_TYPE, NULL, NULL, source_p->host)) == NULL)
  {
    exit_client(source_p, &me, "No CGI:IRC access");
    ilog(L_NOTICE, "INVALID WEBIRC (no host) from %s for %s (%s)", source_p->host, hostname, address);
    return;
  }

  aconf = (struct AccessItem *)map_to_conf(conf);

  if (match_conf_password(password, aconf))
  {
    ilog(L_NOTICE, "WEBIRC from %s for %s (%s)", source_p->host, hostname, address);

    strlcpy(client_p->host, hostname, HOSTLEN + 1);
    strlcpy(client_p->sockhost, address, HOSTIPLEN + 1);
  }
  else
  {
    exit_client(source_p, &me, "No CGI:IRC access");
    ilog(L_NOTICE, "INVALID WEBIRC (bad password) from %s for %s (%s)", source_p->host, hostname, address);
  }
}
