#include <stdlib.h>
#ifndef lint
#ifdef __unused
__unused
#endif
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.37 2003/02/12 18:03:55 davidc Exp $";
#endif
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
#if defined(__cplusplus) || __STDC__
static int yygrowstack(void);
#else
static int yygrowstack();
#endif
#define YYPREFIX "yy"
#line 26 "ircd_parser.y"

#define YY_NO_UNPUT
#include <sys/types.h>

#include "stdinc.h"
#include "ircd.h"
#include "tools.h"
#include "list.h"
#include "s_conf.h"
#include "event.h"
#include "s_log.h"
#include "client.h"	/* for UMODE_ALL only */
#include "pcre.h"
#include "irc_string.h"
#include "irc_getaddrinfo.h"
#include "sprintf_irc.h"
#include "memory.h"
#include "modules.h"
#include "s_serv.h" /* for CAP_LL / IsCapable */
#include "hostmask.h"
#include "send.h"
#include "listener.h"
#include "resv.h"
#include "numeric.h"
#include "s_user.h"

#ifdef HAVE_LIBCRYPTO
#include <openssl/rsa.h>
#include <openssl/bio.h>
#include <openssl/pem.h>
#endif

static char *class_name = NULL;
static struct ConfItem *yy_conf = NULL;
static struct AccessItem *yy_aconf = NULL;
static struct MatchItem *yy_match_item = NULL;
static struct ClassItem *yy_class = NULL;
static char *yy_class_name = NULL;

static dlink_list col_conf_list  = { NULL, NULL, 0 };
static dlink_list hub_conf_list  = { NULL, NULL, 0 };
static dlink_list leaf_conf_list = { NULL, NULL, 0 };
static unsigned int listener_flags = 0;
static unsigned int regex_ban = 0;
static char userbuf[IRCD_BUFSIZE];
static char hostbuf[IRCD_BUFSIZE];
static char reasonbuf[REASONLEN + 1];
static char gecos_name[REALLEN * 4];

extern dlink_list gdeny_items; /* XXX */

static char *resv_reason = NULL;
static char *listener_address = NULL;
static int not_atom = 0;

struct CollectItem
{
  dlink_node node;
  char *name;
  char *user;
  char *host;
  char *passwd;
  int  port;
  int  flags;
#ifdef HAVE_LIBCRYPTO
  char *rsa_public_key_file;
  RSA *rsa_public_key;
#endif
};

static void
free_collect_item(struct CollectItem *item)
{
  MyFree(item->name);
  MyFree(item->user);
  MyFree(item->host);
  MyFree(item->passwd);
#ifdef HAVE_LIBCRYPTO
  MyFree(item->rsa_public_key_file);
#endif
  MyFree(item);
}

static void
unhook_hub_leaf_confs(void)
{
  dlink_node *ptr;
  dlink_node *next_ptr;
  struct CollectItem *yy_hconf;
  struct CollectItem *yy_lconf;

  DLINK_FOREACH_SAFE(ptr, next_ptr, hub_conf_list.head)
  {
    yy_hconf = ptr->data;
    dlinkDelete(&yy_hconf->node, &hub_conf_list);
    free_collect_item(yy_hconf);
  }

  DLINK_FOREACH_SAFE(ptr, next_ptr, leaf_conf_list.head)
  {
    yy_lconf = ptr->data;
    dlinkDelete(&yy_lconf->node, &leaf_conf_list);
    free_collect_item(yy_lconf);
  }
}

#line 134 "ircd_parser.y"
typedef union {
  int number;
  char *string;
} YYSTYPE;
#line 136 "y.tab.c"
#define YYERRCODE 256
#define ACCEPT_PASSWORD 257
#define ACTION 258
#define ADMIN 259
#define AFTYPE 260
#define T_ALLOW 261
#define ANTI_NICK_FLOOD 262
#define ANTI_SPAM_EXIT_MESSAGE_TIME 263
#define AUTOCONN 264
#define T_BLOCK 265
#define BURST_AWAY 266
#define BURST_TOPICWHO 267
#define BYTES 268
#define KBYTES 269
#define MBYTES 270
#define GBYTES 271
#define TBYTES 272
#define CALLER_ID_WAIT 273
#define CAN_FLOOD 274
#define CAN_IDLE 275
#define CHANNEL 276
#define CIDR_BITLEN_IPV4 277
#define CIDR_BITLEN_IPV6 278
#define CIPHER_PREFERENCE 279
#define CLASS 280
#define COMPRESSED 281
#define COMPRESSION_LEVEL 282
#define CONNECT 283
#define CONNECTFREQ 284
#define CRYPTLINK 285
#define DEFAULT_CIPHER_PREFERENCE 286
#define DEFAULT_FLOODCOUNT 287
#define DEFAULT_SPLIT_SERVER_COUNT 288
#define DEFAULT_SPLIT_USER_COUNT 289
#define DENY 290
#define DESCRIPTION 291
#define DIE 292
#define DISABLE_AUTH 293
#define DISABLE_FAKE_CHANNELS 294
#define DISABLE_HIDDEN 295
#define DISABLE_LOCAL_CHANNELS 296
#define DISABLE_REMOTE_COMMANDS 297
#define DOT_IN_IP6_ADDR 298
#define DOTS_IN_IDENT 299
#define DURATION 300
#define EGDPOOL_PATH 301
#define EMAIL 302
#define ENABLE 303
#define ENCRYPTED 304
#define EXCEED_LIMIT 305
#define EXEMPT 306
#define FAILED_OPER_NOTICE 307
#define FAKENAME 308
#define IRCD_FLAGS 309
#define FLATTEN_LINKS 310
#define FFAILED_OPERLOG 311
#define FKILLLOG 312
#define FKLINELOG 313
#define FGLINELOG 314
#define FIOERRLOG 315
#define FOPERLOG 316
#define FOPERSPYLOG 317
#define FUSERLOG 318
#define GECOS 319
#define GENERAL 320
#define GLINE 321
#define GLINES 322
#define GLINE_EXEMPT 323
#define GLINE_LOG 324
#define GLINE_TIME 325
#define GLINE_MIN_CIDR 326
#define GLINE_MIN_CIDR6 327
#define GLOBAL_KILL 328
#define IRCD_AUTH 329
#define NEED_IDENT 330
#define HAVENT_READ_CONF 331
#define HELP_CHANNEL 332
#define HIDDEN 333
#define HIDDEN_ADMIN 334
#define HIDDEN_NAME 335
#define HIDDEN_OPER 336
#define HIDE_SERVER_IPS 337
#define HIDE_SERVERS 338
#define HIDE_SPOOF_IPS 339
#define HOST 340
#define HUB 341
#define HUB_MASK 342
#define IDLETIME 343
#define IGNORE_BOGUS_TS 344
#define INVISIBLE_ON_CONNECT 345
#define IP 346
#define KILL 347
#define KILL_CHASE_TIME_LIMIT 348
#define KLINE 349
#define KLINE_EXEMPT 350
#define KLINE_REASON 351
#define KLINE_WITH_REASON 352
#define KNOCK_DELAY 353
#define KNOCK_DELAY_CHANNEL 354
#define LAZYLINK 355
#define LEAF_MASK 356
#define LINKS_DELAY 357
#define LISTEN 358
#define T_LOG 359
#define LOGGING 360
#define LOG_LEVEL 361
#define MAX_ACCEPT 362
#define MAX_BANS 363
#define MAX_CHANS_PER_USER 364
#define MAX_GLOBAL 365
#define MAX_IDENT 366
#define MAX_LOCAL 367
#define MAX_NICK_CHANGES 368
#define MAX_NICK_TIME 369
#define MAX_NUMBER 370
#define MAX_TARGETS 371
#define MESSAGE_LOCALE 372
#define MIN_NONWILDCARD 373
#define MIN_NONWILDCARD_SIMPLE 374
#define MODULE 375
#define MODULES 376
#define NAME 377
#define NEED_PASSWORD 378
#define NETWORK_DESC 379
#define NETWORK_NAME 380
#define NICK 381
#define NICK_CHANGES 382
#define NO_CREATE_ON_SPLIT 383
#define NO_JOIN_ON_SPLIT 384
#define NO_OPER_FLOOD 385
#define NO_TILDE 386
#define NOT 387
#define NUMBER 388
#define NUMBER_PER_IDENT 389
#define NUMBER_PER_CIDR 390
#define NUMBER_PER_IP 391
#define NUMBER_PER_IP_GLOBAL 392
#define OPERATOR 393
#define OPERS_BYPASS_CALLERID 394
#define OPER_LOG 395
#define OPER_ONLY_UMODES 396
#define OPER_PASS_RESV 397
#define OPER_SPY_T 398
#define OPER_UMODES 399
#define JOIN_FLOOD_COUNT 400
#define JOIN_FLOOD_TIME 401
#define PACE_WAIT 402
#define PACE_WAIT_SIMPLE 403
#define PASSWORD 404
#define PATH 405
#define PING_COOKIE 406
#define PING_TIME 407
#define PING_WARNING 408
#define PORT 409
#define QSTRING 410
#define QUIET_ON_BAN 411
#define REASON 412
#define REDIRPORT 413
#define REDIRSERV 414
#define REGEX_T 415
#define REHASH 416
#define TREJECT_HOLD_TIME 417
#define REMOTE 418
#define REMOTEBAN 419
#define RESTRICT_CHANNELS 420
#define RESTRICTED 421
#define RSA_PRIVATE_KEY_FILE 422
#define RSA_PUBLIC_KEY_FILE 423
#define SSL_CERTIFICATE_FILE 424
#define RESV 425
#define RESV_EXEMPT 426
#define SECONDS 427
#define MINUTES 428
#define HOURS 429
#define DAYS 430
#define WEEKS 431
#define SENDQ 432
#define SEND_PASSWORD 433
#define SERVERHIDE 434
#define SERVERINFO 435
#define SERVLINK_PATH 436
#define SERVICES_NAME 437
#define IRCD_SID 438
#define TKLINE_EXPIRE_NOTICES 439
#define T_SHARED 440
#define T_CLUSTER 441
#define T_SNOOP 442
#define TYPE 443
#define SHORT_MOTD 444
#define SILENT 445
#define SPOOF 446
#define SPOOF_NOTICE 447
#define STATS_E_DISABLED 448
#define STATS_I_OPER_ONLY 449
#define STATS_K_OPER_ONLY 450
#define STATS_O_OPER_ONLY 451
#define STATS_P_OPER_ONLY 452
#define TBOOL 453
#define TMASKED 454
#define T_REJECT 455
#define TS_MAX_DELTA 456
#define TS_WARN_DELTA 457
#define TWODOTS 458
#define T_ALL 459
#define T_BOTS 460
#define T_SOFTCALLERID 461
#define T_CALLERID 462
#define T_CCONN 463
#define T_CCONN_FULL 464
#define T_CLIENT_FLOOD 465
#define T_DEAF 466
#define T_DEBUG 467
#define T_DRONE 468
#define T_EXTERNAL 469
#define T_FULL 470
#define T_INVISIBLE 471
#define T_IPV4 472
#define T_IPV6 473
#define T_LOCOPS 474
#define T_LOGPATH 475
#define T_L_CRIT 476
#define T_L_DEBUG 477
#define T_L_ERROR 478
#define T_L_INFO 479
#define T_L_NOTICE 480
#define T_L_TRACE 481
#define T_L_WARN 482
#define T_MAX_CLIENTS 483
#define T_NCHANGE 484
#define T_OPERWALL 485
#define T_REJ 486
#define T_SERVNOTICE 487
#define T_SKILL 488
#define T_SPY 489
#define T_SSL 490
#define T_UMODES 491
#define T_UNAUTH 492
#define T_UNRESV 493
#define T_UNXLINE 494
#define T_WALLOP 495
#define THROTTLE_TIME 496
#define TOPICBURST 497
#define TRUE_NO_OPER_FLOOD 498
#define TKLINE 499
#define TXLINE 500
#define TRESV 501
#define UNKLINE 502
#define USER 503
#define T_USERMODE 504
#define USE_EGD 505
#define USE_EXCEPT 506
#define USE_INVEX 507
#define USE_KNOCK 508
#define USE_LOGGING 509
#define USE_WHOIS_ACTUALLY 510
#define VHOST 511
#define VHOST6 512
#define XLINE 513
#define WARN 514
#define WARN_NO_NLINE 515
#define WEBIRC 516
const short yylhs[] = {                                        -1,
    0,    0,    5,    5,    5,    5,    5,    5,    5,    5,
    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
    5,    5,    5,    5,    5,    2,    2,    1,    1,    1,
    1,    1,    1,    4,    4,    3,    3,    3,    3,   25,
   27,   27,   28,   28,   28,   29,   30,   13,   31,   31,
   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,
   32,   32,   32,   32,   45,   42,   33,   44,   36,   37,
   38,   39,   40,   34,   43,   41,   35,    6,   46,   46,
   47,   47,   47,   47,   48,   50,   49,    7,   51,   51,
   52,   52,   52,   52,   52,   52,   52,   52,   52,   52,
   52,   52,   52,   53,   54,   57,   64,   58,   62,   59,
   60,   63,   61,   55,   55,   55,   55,   55,   55,   55,
   56,   66,    8,   65,   65,   67,   67,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,
   69,   69,   69,   69,   69,   69,   69,   69,   69,   69,
   69,   70,   68,   71,   72,   90,   91,   76,   93,   75,
   94,   94,   95,   95,   95,   95,   95,   95,   95,   95,
   95,   95,   95,   95,   95,   95,   95,   95,   95,   95,
   95,   77,   78,   84,   79,   80,   81,   82,   83,   85,
   86,   87,   73,   74,   88,   89,   96,   92,   97,   97,
  100,   98,  101,   98,   99,   99,   99,   99,   99,   99,
   99,   99,   99,   99,   99,   99,   99,   99,   99,   99,
   99,  102,   26,  103,  103,  104,  104,  104,  105,  106,
  108,   10,  107,  107,  109,  109,  111,  111,  111,  111,
  111,  111,  111,  111,  111,  111,  111,  111,  111,  111,
  112,  110,  115,  116,  118,  119,  120,  121,  122,  123,
  124,  113,  114,  117,  125,   11,  128,  127,  129,  129,
  130,  130,  126,  126,  131,  131,  131,  131,  131,  136,
  132,  135,  135,  137,  137,  133,  134,  138,   12,  139,
  139,  140,  140,  140,  140,  140,  140,  140,  140,  140,
  140,  140,  140,  140,  140,  140,  140,  140,  141,  142,
  151,  143,  156,  157,  144,  158,  158,  161,  159,  162,
  159,  160,  160,  160,  160,  160,  160,  160,  160,  160,
  160,  145,  146,  147,  154,  148,  149,  150,  152,  153,
  155,  163,   15,  164,  164,  165,  165,  165,  165,  166,
  167,  168,  169,   16,  170,  170,  171,  171,  171,  171,
  172,  173,  175,  174,  176,  176,  177,  177,  177,  177,
  177,  177,  177,  177,  177,  177,  177,  177,  178,   17,
  179,  179,  180,  180,  180,  181,  183,  182,  184,  184,
  185,  185,  185,  185,  185,  185,  185,  185,  185,  185,
  185,  187,   18,  186,  186,  188,  188,  190,  190,  190,
  190,  190,  190,  190,  190,  190,  190,  190,  190,  190,
  190,  190,  190,  190,  190,  190,  190,  191,  189,  192,
  193,  194,  195,  197,  196,  196,  198,  210,  199,  211,
  211,  214,  212,  215,  212,  213,  213,  213,  213,  213,
  213,  207,  204,  206,  205,  203,  209,  200,  201,  202,
  208,  216,   19,  219,  218,  220,  220,  221,  217,  217,
  222,  222,  222,  222,  223,  224,  225,   20,  226,  226,
  227,  227,  227,  228,  229,   21,  230,  230,  231,  231,
  232,  233,   24,  236,  235,  237,  237,  238,  234,  234,
  239,  239,  239,  239,  240,  241,   22,  242,  242,  243,
  243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
  243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
  243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
  243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
  243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
  243,  243,  243,  243,  243,  294,  295,  292,  296,  297,
  293,  254,  244,  245,  283,  246,  247,  248,  249,  250,
  251,  252,  253,  288,  255,  256,  257,  258,  298,  260,
  265,  261,  261,  263,  263,  262,  277,  278,  264,  266,
  267,  268,  269,  271,  270,  259,  273,  282,  284,  285,
  274,  275,  290,  291,  287,  299,  276,  300,  300,  301,
  301,  301,  301,  301,  301,  301,  301,  301,  301,  301,
  301,  301,  301,  301,  301,  301,  301,  301,  302,  272,
  303,  303,  304,  304,  304,  304,  304,  304,  304,  304,
  304,  304,  304,  304,  304,  304,  304,  304,  304,  304,
  304,  280,  281,  279,  286,  289,  305,   23,  306,  306,
  307,  307,  307,  307,  307,  307,  307,  308,  309,  314,
  310,  315,  315,  316,  316,  311,  312,  317,  313,  318,
  318,  319,  319,    9,  320,  320,  321,  321,  321,  321,
  321,  321,  321,  321,  321,  321,  321,  321,  321,  321,
  321,  321,  321,  321,  321,  339,  334,  322,  323,  324,
  325,  327,  328,  329,  330,  326,  331,  332,  333,  335,
  336,  337,  338,   14,  340,  340,  341,  341,  341,  341,
  341,  341,  341,  341,  342,  343,  347,  344,  346,  345,
  348,
};
const short yylen[] = {                                         2,
    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    2,    2,    0,    1,    2,    3,    3,
    3,    3,    3,    0,    1,    2,    3,    3,    3,    5,
    2,    1,    1,    1,    2,    4,    4,    5,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    2,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    5,    2,    1,
    1,    1,    1,    2,    4,    4,    4,    5,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    2,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    0,    7,    0,    1,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    2,    4,    1,    4,    4,    4,    4,    4,    0,    5,
    3,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    0,    5,    3,    1,
    0,    3,    0,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    0,    6,    2,    1,    1,    1,    2,    4,    4,
    0,    7,    0,    1,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
    4,    1,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    0,    6,    0,    5,    3,    1,
    1,    1,    2,    1,    1,    1,    1,    1,    2,    0,
    5,    3,    1,    1,    3,    4,    4,    0,    6,    2,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    2,    4,    4,
    4,    4,    4,    0,    5,    3,    1,    0,    3,    0,
    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    0,    6,    2,    1,    1,    1,    1,    2,    4,
    4,    4,    0,    6,    2,    1,    1,    1,    1,    2,
    4,    4,    0,    5,    3,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    0,    6,
    2,    1,    1,    1,    2,    4,    0,    5,    3,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    0,    7,    0,    1,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    4,    1,    4,
    4,    4,    4,    4,    4,    4,    4,    0,    5,    3,
    1,    0,    3,    0,    2,    1,    1,    1,    1,    1,
    1,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    0,    6,    0,    5,    3,    1,    1,    2,    1,
    1,    1,    1,    1,    4,    4,    0,    6,    2,    1,
    1,    1,    1,    4,    4,    5,    2,    1,    1,    1,
    4,    0,    6,    0,    5,    3,    1,    1,    2,    1,
    1,    1,    1,    1,    4,    4,    5,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    0,    5,    3,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    0,    5,
    3,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    4,    4,    4,    4,    4,    0,    6,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    4,    4,    0,
    5,    3,    1,    1,    1,    4,    4,    0,    5,    3,
    1,    1,    1,    5,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    5,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    4,    4,    4,    4,    4,    4,
    4,
};
const short yydefred[] = {                                      1,
    0,    0,    0,    0,  231,  402,  477,    0,  492,    0,
  667,  288,  462,  265,    0,    0,  122,  342,    0,    0,
  353,  379,  222,    2,    3,    4,    5,    6,    7,    8,
    9,   10,   11,   12,   13,   14,   15,   16,   17,   18,
   19,   20,   21,   22,   23,   24,   25,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   80,   81,   82,   83,  715,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  696,  697,  698,
  699,  700,  701,  702,  703,  704,  705,  706,  707,  708,
  709,  710,  711,  712,  713,  714,  252,    0,  234,  429,
    0,  405,    0,  490,    0,    0,  488,  489,    0,  565,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  639,    0,  616,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  509,  510,  511,  512,
  513,  514,  515,  516,  517,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,  540,  541,  542,
  543,  544,  545,  546,  547,  548,  549,  550,  551,  552,
  553,  554,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   90,   91,   92,   93,   94,   95,   96,   97,   98,   99,
  100,  101,  102,    0,    0,    0,    0,   42,   43,   44,
  153,    0,  125,    0,  744,    0,    0,    0,    0,    0,
    0,    0,    0,  736,  737,  738,  739,  740,  741,  742,
  743,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   50,   51,   52,   53,
   54,   55,   56,   57,   58,   59,   60,   61,   62,   63,
    0,    0,    0,   84,    0,    0,    0,    0,   79,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  695,    0,
    0,  483,    0,    0,    0,  480,  481,  482,    0,    0,
  487,  504,    0,    0,  494,    0,  503,  500,  501,  502,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  508,  677,  688,    0,
    0,  680,    0,    0,    0,  670,  671,  672,  673,  674,
  675,  676,    0,    0,    0,    0,    0,  314,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  291,  292,  293,  294,  295,  296,  297,  298,  299,  300,
  301,  302,  303,  304,  305,  306,  307,  474,    0,  464,
    0,    0,  473,  470,  471,  472,    0,  267,    0,    0,
    0,    0,  276,  274,  275,  277,  278,  103,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   89,   45,    0,    0,    0,   41,    0,    0,    0,
    0,    0,    0,  345,  346,  347,  348,    0,    0,    0,
    0,    0,    0,    0,    0,  735,   64,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   49,    0,    0,  363,    0,    0,  356,  357,  358,
  359,    0,    0,  387,    0,  382,  383,  384,    0,    0,
    0,    0,  225,  226,  227,    0,    0,    0,   78,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  694,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  236,  237,  238,  239,  240,  241,
  242,  243,  244,  245,  246,  247,  248,  249,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  438,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  407,
  408,  409,  410,  411,  412,  413,  414,  415,  416,  417,
  418,  419,  420,  421,  422,  423,  424,  425,  426,    0,
    0,    0,  479,    0,  486,    0,    0,    0,    0,  499,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  507,    0,
    0,    0,    0,    0,    0,    0,  669,  308,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  290,    0,    0,    0,    0,
  469,  279,    0,    0,    0,    0,    0,  273,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   88,    0,    0,   40,
    0,    0,    0,    0,    0,  197,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  159,    0,    0,    0,    0,  127,  128,  129,  130,  131,
  132,  133,  134,  135,  136,  137,  138,  139,  140,  141,
  142,  143,  144,  145,  146,  147,  148,  149,  150,  349,
    0,    0,    0,    0,  344,    0,    0,    0,    0,    0,
    0,    0,  734,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   48,  360,    0,    0,
    0,    0,  355,  385,    0,    0,    0,  381,  228,    0,
    0,    0,  224,   87,   86,   85,  731,  728,  727,  716,
  718,    0,    0,    0,    0,    0,   27,   28,  722,  723,
  726,  724,  729,  730,  732,  733,  725,  717,  719,  720,
  721,  250,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  235,  427,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  406,    0,
    0,  478,  491,    0,    0,    0,  493,  577,  581,  568,
  597,  610,  609,  664,  614,  575,  666,  606,  612,  576,
  566,  567,  584,  573,  605,  574,  587,  572,  586,  585,
  580,  579,  578,  607,  604,  662,  663,  601,  598,  643,
  659,  660,  644,  645,  646,  647,  654,  648,  657,  661,
  650,  655,  651,  656,  649,  653,  652,  658,    0,  642,
  603,  620,  636,  637,  621,  622,  623,  624,  631,  625,
  634,  638,  627,  632,  628,  633,  626,  630,  629,  635,
    0,  619,  596,  599,  613,  570,  608,  571,  600,  589,
  594,  595,  592,  593,  590,  591,  583,  582,    0,    0,
    0,   35,   36,  665,  615,  602,  611,  569,  588,    0,
    0,    0,    0,    0,    0,  668,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  289,    0,    0,    0,  463,    0,    0,    0,
    0,    0,  283,  266,  107,  113,  111,  110,  112,  108,
  109,  106,  114,  120,  115,  119,  117,  118,  116,  105,
  104,  121,   46,   47,  151,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  126,
    0,    0,    0,  343,  750,  745,  749,  747,  751,  746,
  748,   69,   73,   77,   67,   71,   70,   66,   65,   72,
   68,   76,   74,   75,    0,    0,    0,  354,    0,    0,
  380,    0,    0,  223,   29,   30,   31,   32,   33,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  232,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  403,  484,  485,  505,  506,  498,    0,
  497,  640,    0,  617,    0,   37,   38,   39,  693,  692,
    0,  691,  679,  678,  685,  684,    0,  683,  687,  686,
  335,  312,  313,  334,  318,    0,  317,    0,  337,  333,
  332,  341,  336,  310,  340,  339,  338,  311,  309,  476,
  468,    0,  467,  475,  272,  271,    0,  270,  287,  286,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  123,  351,  352,  350,
  361,  367,  373,  378,  376,  375,  372,  368,  371,  374,
  369,  377,  370,    0,  366,  362,  386,  391,  397,  401,
  400,  399,  396,  392,  395,  398,  393,  394,    0,  390,
  229,  230,  262,  263,  256,  258,  260,  259,  257,  251,
  264,  255,  253,  254,  261,  433,  435,  436,  456,  461,
  460,  455,  454,  453,  437,  442,    0,  441,    0,  430,
  458,  459,  428,  434,  452,  432,  457,  431,  495,    0,
  641,  618,  689,    0,  681,    0,    0,  315,    0,  326,
  327,  323,  329,  325,  324,  331,  328,  330,  322,  321,
  465,    0,  268,    0,  285,  282,  281,  192,  158,  190,
  156,  201,    0,  200,    0,  188,  182,  193,  194,  185,
  152,  189,  155,  191,  183,  184,  157,  196,  195,  163,
  179,  180,  164,  165,  166,  167,  174,  168,  177,  181,
  170,  175,  171,  176,  169,  173,  172,  178,    0,  162,
  187,  154,  186,  364,    0,  388,    0,    0,  439,    0,
  449,  450,  447,  448,  446,  451,  445,  496,  690,  682,
  319,  316,  466,  269,    0,  198,    0,  213,  211,  220,
  210,  205,  214,  218,  207,  215,  217,  212,  206,  219,
  221,  216,  208,  209,  204,  160,    0,  365,  389,  443,
  440,  202,  199,  161,
};
const short yydgoto[] = {                                       1,
  917,  918, 1072, 1073,   24,   25,   26,   27,   28,   29,
   30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
   40,   41,   42,   43,   44,   45,  277,  278,  279,  280,
  316,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   73,   74,   75,   76,   77,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,  282,   62,  835,  283,  836,  837,
  838,  839,  840,  841,  842,  843,  844,  845,  846,  847,
  848,  849,  850,  851,  852,  853,  854,  855,  856,  857,
  858,  859, 1155, 1459, 1460, 1140, 1423, 1424, 1505, 1485,
 1425,   68,  582,  583,  584,  585,  118,   50,  624,  119,
  625,  626,  627,  628,  629,  630,  631,  632,  633,  634,
  635,  636,  637,  638,   59,  502,  503,  783, 1287, 1288,
  504,  505,  506,  507, 1112, 1293, 1113,   57,  470,  471,
  472,  473,  474,  475,  476,  477,  478,  479,  480,  481,
  482,  483,  484,  485,  486,  487,  763, 1266, 1267, 1410,
 1397, 1268,   63,  533,  534,  535,  536,  537,   66,  567,
  568,  569,  570,  571,  890, 1334, 1335,   67,  575,  576,
  577,  578,  896, 1349, 1350,  121,   51,  659,  122,  660,
  661,  662,  663,  664,  665,  666,  667,  668,  669,  670,
  671,  672,  673,  674,  675,  676,  677,  678,  679,  958,
 1377, 1378, 1477, 1468, 1379,   58,  492,  493,  778, 1282,
 1283,  494,  495,  496,   52,  365,  366,  367,  368,  126,
  127,  128,   54,  376,  377,  688, 1240, 1241,  378,  379,
  380,  186,  187,  188,  189,  190,  191,  192,  193,  194,
  195,  196,  197,  198,  199,  200,  201,  202,  203,  204,
  205,  206,  207,  208,  209,  210,  211,  212,  213,  214,
  215,  216,  217,  218,  219,  220,  221,  222,  223,  224,
  225,  226,  227,  228,  229,  230,  231,  232,  233,  234,
  235,  236,  237,  238,  239,  240,  241,  242,  415, 1051,
 1052,  413, 1029, 1030,   56,  445,  446,  447,  448,  449,
  450,  451,  452,  753, 1257, 1258,  750, 1251, 1252,   97,
   98,   99,  100,  101,  102,  103,  104,  105,  106,  107,
  108,  109,  110,  111,  112,  113,  114,  115,  116,  293,
  294,  295,  296,  297,  298,  299,  300,  301,
};
const short yysindex[] = {                                      0,
  320,  -40,  -95,  -77,    0,    0,    0,  -74,    0,  -69,
    0,    0,    0,    0,  -66,  -60,    0,    0,  -55,  -53,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -247,  720, -386,
 -314,  -22, -222,   -2,  425,    8,   10,   15,   25, -205,
 -236, -287,   39,  234,  121,   59,   68,   70,  100,  117,
  140,  142, -119,    0,    0,    0,    0,    0,  149,  150,
  157,  159,  163,  167,  177,  180,  183,  184,  187,  199,
  204,  210,  236,  239,  242,  244,  111,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  188,    0,    0,
  189,    0, -231,    0,  245, -103,    0,    0, -216,    0,
  252,  257,  258,  259,  264,  270,  280,  281,  285,  286,
  287,  289,  292,  293,  294,  296,  297,  298,  300,  301,
  302,  303,  304,  308,  310,  311,  312,  318,  324,  328,
  334,  335,    0,  337,    0,  340,  342,  352,  353,  355,
  356,  364,  367,  368,  369,  370,  380,  381,  382,  383,
  384,  388,  391,  396,  400, -122,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -241,  715, -246,   51,  404,  405,  406,  407,
  412,  415,  416,  418,  419,  420,  421,  422,  424,  -25,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  428,  427,  430, -109,    0,    0,    0,
    0,  317,    0, -226,    0,  435,  436,  441,  442,  443,
  444,  448,  213,    0,    0,    0,    0,    0,    0,    0,
    0,  451,  452,  454,  455,  457,  458,  459,  460,  463,
  464,  465,  466,  467,  469,  -56,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -209, -227, -224,    0,   79,  123,  124,  479,    0,   96,
  164,  166,  102,  103,  169,  169,  174,  176,  112,  113,
  185,  169,  115,  131,  133,  134,  135,  524,    0,  170,
  646,    0,  528,  532, -114,    0,    0,    0,  191,  535,
    0,    0,  534,  536,    0, -104,    0,    0,    0,    0,
  154,  169,  155,  169,  221,  201,  224,  160,  161,  162,
  228,  211,  172,  232,  240,  241,  181,  169,  190,  192,
  247,  220,  193,  248,  249,  169,  250,  238,  256,  262,
  198,  202,  591,  203,  604,  169,  169,  217,  169,  261,
  219,  222,  223, -418, -280,  226,  229,  169,  169,  278,
  169,  230,  237,  251,  253,  609,    0,    0,    0,  613,
  616,    0,  628,  631,  -44,    0,    0,    0,    0,    0,
    0,    0,  614,  633,  634,  636,  639,    0,  642,  644,
  647,  649,  654,  655,  656,  658,  659,  660,  664,  128,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  666,    0,
  667, -111,    0,    0,    0,    0,  626,    0,  669,  670,
  672, -107,    0,    0,    0,    0,    0,    0,  325,  326,
  327,  329,  330,  331,  332,  333,  182,  336,  338,  291,
  675,    0,    0,  339,  343,  679,    0,  559,  691,  697,
  698,  701, -113,    0,    0,    0,    0,  313,  314,  319,
  361,  321,  322,  169,  719,    0,    0,  372,  375,  347,
  376,  378,  385,  392,  393,  394,  398,  401,  399,  402,
  721,    0,  731,  718,    0,  730,  -99,    0,    0,    0,
    0,  733,  740,    0,  -98,    0,    0,    0,  747,  750,
  752, -121,    0,    0,    0,  755,  757,  758,    0,  761,
  764,  766,  770,  771,   -8,  773,  774,  775,  776,  778,
  779,  781,  782,  784,  785,  786,  787,  788,    0,  789,
  791,  792,  793,  794,  795,  796,  797,  798,  799,  801,
  804,  809,  810,  215,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  790,  811,
  818,  823,  824,  825,  827,  828,  830,  831,    0,  833,
  835,  836,  837,  838,  839,  840,  843,  844,  130,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  456,
  497,  850,    0,  852,    0,  502,  503,  853,  856,    0,
  858,  859,  860,  861,  863,  865,  870,  873,  874,  875,
  878,  879,  880,  883,  884,  885,  887,  888,  889,  890,
  892,  894,  897,  898,  900,  903,  905,  906,  907,  908,
  909,  910,  722,  911,  769,  913,  915,  920,  921,  922,
  924,  925,  926,  931,  932,  933,  934,  935,  937,  938,
  939, -204,  940,  941,  944,  945,  946,  948,    0,  949,
  169,  558,  951,  603,  605,  958,    0,    0,  565,  611,
  572,  574,  967,  576,  577,  578,  579,  580,  625,  648,
  629,  637,  581,  638,  982,    0,  641,  985,  643,  990,
    0,    0,  991,  650,  653,  668,  995,    0,  998,  999,
 1000, 1005, 1008, 1009, 1011, 1016, 1017, 1018, 1019, 1021,
 1022, 1023, 1026, 1028, 1029, 1030,    0, 1031, 1032,    0,
 1033, 1034, 1035, 1036, 1037,    0, 1038, 1039, 1046, 1047,
 1049, 1054, 1056, 1057, 1061, 1062, 1063, 1066, 1069, 1071,
    0, 1072, 1073, 1074,  -67,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  684,  728,  729, 1082,    0, 1083, 1085, 1086, 1087, 1088,
 1089, 1090,    0, 1091, 1093, 1094, 1095, 1096, 1097, 1099,
 1100, 1101, 1104, 1105, 1106, 1107,    0,    0,  759, 1111,
  763, 1108,    0,    0,  765, 1113, 1109,    0,    0,  777,
  802, 1121,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  169,  169,  169,  169,  169,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  806,  807,  169,  812,  813,  814,  815,  803,
  816,  817,  169,  169,  278, 1122,    0,    0,  832, -285,
  744,  834,  841,  745,  746,  762,  842, 1155,  849,  867,
  868,  871,  846,  872,  876,  767,  877, 1160,    0, 1162,
 1163,    0,    0, 1164, 1165,  822,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  -21,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   -3,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  278,  278,
  278,    0,    0,    0,    0,    0,    0,    0,    0, -260,
 1166, 1182, -257, 1186, 1187,    0, 1188, 1189, 1190, 1191,
  886, 1201, 1203, 1204, 1211, 1221, 1224, 1225, 1226, 1229,
 1236, 1237,    0, 1239,  891, 1240,    0, -320, 1242, 1243,
  845, 1260,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  854,  895,  855,  857, 1248,
  862,  864,  866,  869,  881,  901,  882,  902,  893,  896,
  899,  904,  912,  914, 1252,  916,  917,  918, 1257,    0,
 1259, 1261, 1262,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1264,  612, 1265,    0, 1266,  677,
    0, 1267, 1269,    0,    0,    0,    0,    0,    0, 1270,
 1271, 1272, 1273, 1274, 1277, 1278, 1279, 1280, 1281, 1282,
 1283, 1284,    0, 1285, 1286, 1288, 1289, 1291, 1292, 1294,
 1295, 1296, 1297,  970, 1299, 1300, 1301, 1302, 1303, 1304,
 1305, 1307, 1309,    0,    0,    0,    0,    0,    0,    1,
    0,    0,  722,    0,  769,    0,    0,    0,    0,    0,
   23,    0,    0,    0,    0,    0,   31,    0,    0,    0,
    0,    0,    0,    0,    0,   33,    0, -232,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   35,    0,    0,    0,    0,   36,    0,    0,    0,
  984,  668, 1311, 1314, 1315, 1316, 1317,  992, 1318, 1319,
 1321, 1322, 1323, 1324, 1325, 1326, 1327, 1328, 1329, 1330,
 1331, 1332,  805, 1333, 1334, 1335,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   43,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   44,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   45,    0, -233,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  822,
    0,    0,    0, -260,    0, -257, -232,    0,  886,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  891,    0, -320,    0,    0,    0,    0,    0,    0,
    0,    0,   55,    0,  365,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   61,    0,
    0,    0,    0,    0,  612,    0,  677, -233,    0,  970,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  365,    0,  992,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  805,    0,    0,    0,
    0,    0,    0,    0,
};
const short yyrindex[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1275,
 1276,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1287,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1336,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1337,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1336, 1336, 1336, 1336, 1336,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1337, 1337,
 1337,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  379,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   73, 1338,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -188,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  624,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  379,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -188,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  624,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,
};
const short yygindex[] = {                                      0,
 -345, -787, -428, -863,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1123,    0,    0,
    0, 1092,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1339,    0,    0,    0,
    0, 1141,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  567,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -102,    0,    0,  -84,  -81,    0,
    0,    0,    0,  829,    0,    0,    0,    0,    0,    0,
  783,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   -5,
  919,    0,    0,    0,    0,    0,  114,    0,    0,  943,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   16,   17,
    0,    0,    0,    0,  923,    0,    0,    0,    0,    0,
  851,    0,    0,    0,    0,    0,  -49,    0,    0,  847,
    0,    0,    0,    0,  -50,    0,    0,    0,    0,  760,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  -47,  -48,    0,    0,    0,    0,    0,    0,    0,
   12,  936,    0,    0,    0,    0, 1060,    0,    0,    0,
 1306,    0,    0,    0,    0,    0,    0,   37, 1050,    0,
    0,    0, 1244,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  186,    0,    0,  194,    0,    0,  988,    0,    0,    0,
    0,    0,    0,    0,    0,   38,    0,    0,   41,    0,
 1341,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1136,    0,    0,    0,    0,    0,    0,    0,
};
#define YYTABLESIZE 1456
const short yytable[] = {                                     596,
  597,  743,  436,  902, 1249,  338,  603, 1255,   69,  488,
  682,  864, 1285,  780,  438,  526,  439,  787,   46,  274,
  689,  370, 1243,  117,  362,  892,  897,   48,  572,  529,
 1471,  579, 1472,  124,  734,  735,  692, 1242,  694,  372,
 1245, 1400, 1401,   70, 1390,   49,  563, 1473,   53,  530,
  247, 1474,  708,   55,   71, 1244,   60, 1159,  440, 1389,
  716,  441,   61, 1069, 1070, 1071, 1394,   64,  561,   65,
  726,  727, 1402,  729, 1396,  444, 1399,  444, 1412, 1414,
  756, 1393,  740,  741,   47,  744, 1465, 1467, 1470, 1395,
 1403, 1398,  444, 1411, 1413,  120,  444, 1404, 1487,  521,
  123, 1464, 1466, 1469, 1507,  248,  249,  250,  251,  252,
  253,  254,  255, 1486,  363,  580,  284, 1405,  442, 1506,
  129, 1475,  281,  125, 1195, 1196, 1197, 1198, 1199,   72,
  243,  284,  244,  130,  579,  443,   69,  245,  275,  131,
  132,  362,  529,  133,  488, 1406,  274,  246,  497,  573,
  134,  372,  124, 1407,  531,  256,  563,  572,  334,  135,
  373,  284,  530,  136,  137,  489,  444,  564,  276, 1286,
  138,   70,  736,  737,  139,  140,  141,  335,  142,  581,
  364,  331,   71,  742,  143,  532, 1215, 1216,  811,  257,
  332,  812,  333, 1408, 1250,  374,  490, 1256,  872,  302,
  336,  498,  337,  144,  145, 1246, 1247, 1248,  146,  340,
  341,  438,  813,  439, 1409,  574,  147,  342,  580,  343,
  148,  149,  150,  344,  814,  151,  375,  345,  152,  153,
  247,  363,  499,  565,  303,  358,  815,  346,  500,  154,
  347,  816,  125,  348,  349,  155,  156,  350,  157,  158,
  159,  160,  775,  817,  968,  440,  491,   72,  441,  351,
  818,  444,  161, 1476,  352,  275,  819,  531,  820,  258,
  353,  162,  373,  163,  164,  304,  165,  564,  573,  166,
  167,  821,  581,  168,  305,  248,  249,  250,  251,  252,
  253,  254,  255,  566,  169,  276,  354,  364,  532,  355,
  489,  501,  356,  259,  357,  369,  497,  374,  444,  822,
  360,  361,  381,  170,  823,  442,  171,  382,  383,  384,
  306,  172,  307,  308,  385,  173,  174,  175,  176,  177,
  386,  490,  443,  178,  179,  256,  824,  545,  375,  946,
  387,  388,  180,  565,  574,  389,  390,  391,  825,  392,
  826,  827,  393,  394,  395,  828,  396,  397,  398,  498,
  399,  400,  401,  402,  403,  309,   78,  310,  404,  257,
  405,  406,  407,  181,  829,  182,  302,   79,  408,  595,
  311,  312,  183,  453,  409,  639,  640,  184,  410,  641,
  499,  491,  185,  642,  411,  412,  500,  414,   80,   81,
  416,  454,  417,  566,   82, 1081,   83,  455,  643,  644,
  645,  303,  418,  419,  646,  420,  421,  830,  912,  913,
  914,  915,  916,  831,  422,  610,  313,  423,  424,  425,
  426,  456,  457,  647,  832,  833,  458,  648,  649,  528,
  427,  428,  429,  430,  431,  834,  611,  612,  432,  258,
  459,  433,  304,  613,  314,  315,  434,  460,  444,  501,
  435,  305,  508,   84,   85,  509,  510,  511,  285,  650,
  610,  651,  512,   86,   87,  513,  514,  461,  515,  516,
  517,  518,  519,  259,  520,  652,  523,  524,  586,  285,
  525,  611,  612,   88,   89,  538,  539,  306,  613,  307,
  308,  540,  541,  542,  543,  462,  653,  286,  544,  547,
   90,   91,  548,  463,  549,  550, 1212,  551,  552,  553,
  554,   92,  287,  555,  556,  557,  558,  559,  286,  560,
   93,  464,  587,  588,  614,  615,  616,  589,  654,  617,
  465,  466,  309,  287,  310,  288,  618,  289,  590,  290,
  291,  591,  655,  592,  593,  594,  595,  311,  312,  619,
  620,  598,  656,  599,  600,  601,  288,  604,  289,  292,
  290,  291,  602,  467,  468,    2,  621,  622,    3,  614,
  615,  616,  609,  605,  617,  606,  607,  608,  680, 1202,
  292,  618,  681,  685,  686,    4,  687, 1210, 1211,    5,
  684,  623,    6,  313,  619,  620,  691,  693,  695,    7,
  696,  697,  698,  699,  700,  701,   94,   95,   96,  704,
  702,  621,  622, 1488,  703,    8,  657,  705,  706,  712,
  469,  314,  315,  707,  711,  714,  715,  717,    9,   10,
  658,   11,  709,  719,  710,  713,  623,  718,   12,  720,
  721,  723,  320,  320,  722,  724, 1489,  797,  798,  799,
  800,  801,  802,  803,  725,  742,   13,  749, 1490,  728,
  730,  731,  758,  751,  732,  733,  752,   14,  738,   15,
  130,  739,  745,  320,  782, 1491,  131,  132,  754,  746,
  133,  755, 1492,  759,  760,   16,  761,  134, 1493,  762,
 1494,  320,  764,  747,  765,  748,  135,  766,  320,  767,
  136,  137,   17, 1495,  768,  769,  770,  138,  771,  772,
  773,  139,  140,  141,  774,  142,  777,  779,  320,  784,
  785,  143,  786,  807,  789,  790,  791,  810,  792,  793,
  794,  795,  796,  806,   18,  804, 1496,  805,  808,  860,
  144,  145,  809,   19,   20,  146,  320,  861,  862,   21,
   22,  863, 1497,  147,  320,  866,  867,  148,  149,  150,
  869,  868,  151,  870,  871,  152,  153,  873,  889,  887,
 1498,  874, 1499, 1500,  875,  877,  154,  878,  884,  888,
  891,  894,  155,  156,  879,  157,  158,  159,  160,  876,
  895,  880,  881,  882,  320,  899, 1501,  883,  885,  161,
  900,  886,  901,  904,  811,  905,  906,  812,  162,  907,
  163,  164,  908,  165,  909,  320,  166,  167,  910,  911,
  168,  919,  920,  921,  922,   23,  923,  924,  813,  925,
  926,  169,  927,  928,  929,  930,  931,  932,  948, 1502,
  814,  933,  934,  935,  936,  937,  938,  939,  940,  941,
  170,  942,  815,  171,  943,  970, 1503,  816,  172,  944,
  945,  949,  173,  174,  175,  176,  177, 1504,  950,  817,
  178,  179,  203,  951,  952,  953,  818,  954,  955,  180,
  956,  957,  819,  959,  820,  960,  961,  962,  963,  964,
  965,  639,  640,  966,  967,  641,  971,  821,  972,  642,
  973,  974,  975,  976,  977,  203,  978,  979,  980,  981,
  181,  982,  182,  983,  643,  644,  645,  203,  984,  183,
  646,  985,  986,  987,  184,  822,  988,  989,  990,  185,
  823,  991,  992,  993,  203,  994,  995,  996,  997,  647,
  998,  203,  999,  648,  649, 1000, 1001,  203, 1002,  203,
 1322, 1003,  824, 1004, 1005, 1006, 1007, 1008, 1009, 1031,
  453, 1053,  203, 1054,  825,   78,  826,  827, 1055, 1056,
 1057,  828, 1058, 1059, 1060,  650,   79,  651,  454, 1061,
 1062, 1063, 1064, 1065,  455, 1066, 1067, 1068, 1074, 1075,
  829,  652, 1076, 1077, 1078,  203, 1079,   80,   81, 1080,
 1082, 1083, 1084,   82, 1085,   83, 1086, 1087,  456,  457,
 1088,  203,  653,  458, 1089, 1338, 1090, 1091, 1092, 1093,
 1094, 1095, 1096, 1101, 1097, 1098, 1323,  459, 1099,  203,
 1103,  203,  203,  830,  460, 1105, 1100, 1102, 1107,  831,
 1104, 1108, 1106, 1114,  654, 1111, 1115, 1116, 1117, 1109,
  832,  833, 1110, 1118,  461,  203, 1119, 1120,  655, 1121,
 1324,  834,   84,   85, 1122, 1123, 1124, 1125,  656, 1126,
 1127, 1128,   86,   87, 1129, 1325, 1130, 1131, 1132, 1133,
 1134, 1135,  462, 1161, 1136, 1137, 1138, 1139, 1141, 1142,
  463, 1339,   88,   89, 1326, 1327, 1143, 1144,  203, 1145,
 1328, 1329, 1330, 1331, 1146, 1332, 1147, 1148,  464,   90,
   91, 1149, 1150, 1151, 1333,  203, 1152,  465,  466, 1153,
   92, 1154, 1156, 1157, 1158, 1340,  203, 1162, 1163,   93,
 1164, 1165,  657, 1166, 1167, 1168, 1169, 1170, 1171, 1172,
 1341, 1173, 1174, 1175, 1176, 1177,  658, 1178, 1179, 1180,
  467,  468, 1181, 1182, 1183, 1184, 1188, 1191, 1185, 1342,
 1343, 1186, 1187, 1190, 1189, 1344, 1345, 1346, 1347, 1194,
 1213, 1010, 1011, 1012, 1013, 1014, 1192, 1015, 1016, 1348,
 1017, 1018, 1019, 1200, 1201, 1020, 1217, 1220, 1221, 1203,
 1204, 1205, 1206, 1208, 1209, 1021, 1022, 1023, 1024, 1025,
 1026, 1193, 1207, 1027, 1222, 1224, 1028,  469, 1234, 1232,
 1235, 1236, 1237, 1238, 1253,   94,   95,   96, 1032, 1033,
 1034, 1035, 1036, 1229, 1037, 1038, 1239, 1039, 1040, 1041,
 1254, 1214, 1042, 1218, 1259, 1260, 1261, 1262, 1263, 1264,
 1219, 1223, 1043, 1044, 1045, 1046, 1047, 1048, 1225, 1269,
 1049, 1270, 1271, 1050, 1440, 1441, 1442, 1443, 1444, 1272,
 1445, 1446, 1265, 1447, 1448, 1449, 1226, 1227, 1450, 1273,
 1228, 1230, 1274, 1275, 1276, 1231, 1233, 1277, 1451, 1452,
 1453, 1454, 1455, 1456, 1278, 1279, 1457, 1280, 1284, 1458,
 1289, 1290, 1291, 1292, 1295, 1281, 1294, 1296, 1298, 1297,
 1304, 1306, 1313, 1310, 1299, 1317, 1300, 1318, 1301, 1319,
 1320, 1302, 1321, 1336, 1337, 1351, 1315, 1352, 1353, 1354,
 1355, 1356, 1357, 1303, 1305, 1358, 1359, 1360, 1361, 1362,
 1363, 1364, 1365, 1366, 1367, 1307, 1368, 1369, 1308, 1370,
 1371, 1309, 1372, 1373, 1374, 1375, 1376, 1380, 1381, 1382,
 1383, 1384, 1385, 1386, 1311, 1387, 1312, 1388, 1314, 1417,
 1316, 1415, 1418, 1419, 1420, 1421, 1426, 1427, 1422, 1428,
 1429, 1430, 1431, 1432, 1433, 1434, 1435, 1436, 1437, 1438,
 1439, 1461, 1462, 1463,   26,   34,  280,  233,  404,  527,
  522, 1160, 1513, 1512, 1514, 1416,  947,  562, 1484,  124,
  903,  339,  776, 1481, 1482, 1508, 1509,  893,  969, 1510,
  788,  898, 1511, 1483,  683,  690, 1478,  781,  546,  437,
 1392,  371,  757, 1480, 1479,    0, 1391,  359,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  865,
};
const short yycheck[] = {                                     345,
  346,  430,  125,  125,  265,  125,  352,  265,  256,  256,
  125,  125,  333,  125,  256,  125,  258,  125,   59,  256,
  125,  125,   44,  410,  256,  125,  125,  123,  256,  256,
  264,  256,  266,  256,  453,  454,  382,   59,  384,  256,
   44,  274,  275,  291,   44,  123,  256,  281,  123,  276,
  256,  285,  398,  123,  302,   59,  123,  125,  300,   59,
  406,  303,  123,  268,  269,  270,   44,  123,  125,  123,
  416,  417,  305,  419,   44,  264,   44,  266,   44,   44,
  125,   59,  428,  429,  125,  431,   44,   44,   44,   59,
  323,   59,  281,   59,   59,  410,  285,  330,   44,  125,
  123,   59,   59,   59,   44,  311,  312,  313,  314,  315,
  316,  317,  318,   59,  346,  340,   44,  350,  360,   59,
  123,  355,  410,  346,  912,  913,  914,  915,  916,  377,
  123,   59,  123,  256,  256,  377,  256,  123,  375,  262,
  263,  256,  256,  266,  256,  378,  256,  123,  256,  377,
  273,  256,  256,  386,  381,  361,  256,  256,   59,  282,
  377,  123,  276,  286,  287,  412,  355,  377,  405,  490,
  293,  291,  453,  454,  297,  298,  299,   61,  301,  404,
  412,  123,  302,  388,  307,  412,  472,  473,  256,  395,
  123,  259,  123,  426,  455,  412,  443,  455,  544,  256,
   61,  309,   61,  326,  327, 1069, 1070, 1071,  331,   61,
   61,  256,  280,  258,  447,  443,  339,   61,  340,   61,
  343,  344,  345,   61,  292,  348,  443,   61,  351,  352,
  256,  346,  340,  443,  291,  125,  304,   61,  346,  362,
   61,  309,  346,   61,   61,  368,  369,   61,  371,  372,
  373,  374,  125,  321,  125,  300,  503,  377,  303,   61,
  328,  503,  385,  497,   61,  375,  334,  381,  336,  475,
   61,  394,  377,  396,  397,  332,  399,  377,  377,  402,
  403,  349,  404,  406,  341,  311,  312,  313,  314,  315,
  316,  317,  318,  503,  417,  405,   61,  412,  412,   61,
  412,  409,   61,  509,   61,   61,  256,  412,  497,  377,
  123,  123,   61,  436,  382,  360,  439,   61,   61,   61,
  377,  444,  379,  380,   61,  448,  449,  450,  451,  452,
   61,  443,  377,  456,  457,  361,  404,  125,  443,  125,
   61,   61,  465,  443,  443,   61,   61,   61,  416,   61,
  418,  419,   61,   61,   61,  423,   61,   61,   61,  309,
   61,   61,   61,   61,   61,  422,  256,  424,   61,  395,
   61,   61,   61,  496,  442,  498,  256,  267,   61,  388,
  437,  438,  505,  256,   61,  256,  257,  510,   61,  260,
  340,  503,  515,  264,   61,   61,  346,   61,  288,  289,
   61,  274,   61,  503,  294,  751,  296,  280,  279,  280,
  281,  291,   61,   61,  285,   61,   61,  485,  427,  428,
  429,  430,  431,  491,   61,  256,  483,   61,   61,   61,
   61,  304,  305,  304,  502,  503,  309,  308,  309,  123,
   61,   61,   61,   61,   61,  513,  277,  278,   61,  475,
  323,   61,  332,  284,  511,  512,   61,  330,  503,  409,
   61,  341,   59,  353,  354,   61,   61,   61,  256,  340,
  256,  342,   61,  363,  364,   61,   61,  350,   61,   61,
   61,   61,   61,  509,   61,  356,   59,   61,  410,  256,
   61,  277,  278,  383,  384,   61,   61,  377,  284,  379,
  380,   61,   61,   61,   61,  378,  377,  295,   61,   59,
  400,  401,   61,  386,   61,   61,  945,   61,   61,   61,
   61,  411,  310,   61,   61,   61,   61,   61,  295,   61,
  420,  404,  410,  410,  365,  366,  367,   59,  409,  370,
  413,  414,  422,  310,  424,  333,  377,  335,  453,  337,
  338,  388,  423,  388,  453,  453,  388,  437,  438,  390,
  391,  388,  433,  388,  453,  453,  333,  453,  335,  357,
  337,  338,  388,  446,  447,  256,  407,  408,  259,  365,
  366,  367,   59,  453,  370,  453,  453,  453,   61,  935,
  357,  377,   61,   59,   61,  276,   61,  943,  944,  280,
  410,  432,  283,  483,  390,  391,  453,  453,  388,  290,
  410,  388,  453,  453,  453,  388,  506,  507,  508,  388,
  410,  407,  408,  259,  453,  306,  497,  388,  388,  410,
  503,  511,  512,  453,  388,  388,  388,  388,  319,  320,
  511,  322,  453,  388,  453,  453,  432,  410,  329,  388,
  453,   61,  274,  275,  453,  453,  292,  476,  477,  478,
  479,  480,  481,  482,   61,  388,  347,   59,  304,  453,
  410,  453,   59,   61,  453,  453,   61,  358,  453,  360,
  256,  453,  453,  305,   59,  321,  262,  263,   61,  453,
  266,   61,  328,   61,   61,  376,   61,  273,  334,   61,
  336,  323,   61,  453,   61,  453,  282,   61,  330,   61,
  286,  287,  393,  349,   61,   61,   61,  293,   61,   61,
   61,  297,  298,  299,   61,  301,   61,   61,  350,   61,
   61,  307,   61,   59,  410,  410,  410,   59,  410,  410,
  410,  410,  410,  453,  425,  410,  382,  410,  410,   59,
  326,  327,  410,  434,  435,  331,  378,   61,   61,  440,
  441,   61,  398,  339,  386,  453,  453,  343,  344,  345,
  410,  453,  348,  453,  453,  351,  352,   59,   61,   59,
  416,  410,  418,  419,  410,  410,  362,  410,  388,   59,
   61,   59,  368,  369,  410,  371,  372,  373,  374,  453,
   61,  410,  410,  410,  426,   59,  442,  410,  410,  385,
   61,  410,   61,   59,  256,   59,   59,  259,  394,   59,
  396,  397,   59,  399,   59,  447,  402,  403,   59,   59,
  406,   59,   59,   59,   59,  516,   59,   59,  280,   59,
   59,  417,   59,   59,   59,   59,   59,   59,   59,  485,
  292,   61,   61,   61,   61,   61,   61,   61,   61,   61,
  436,   61,  304,  439,   61,  410,  502,  309,  444,   61,
   61,   61,  448,  449,  450,  451,  452,  513,   61,  321,
  456,  457,  259,   61,   61,   61,  328,   61,   61,  465,
   61,   61,  334,   61,  336,   61,   61,   61,   61,   61,
   61,  256,  257,   61,   61,  260,  410,  349,   59,  264,
   59,  410,  410,   61,   59,  292,   59,   59,   59,   59,
  496,   59,  498,   59,  279,  280,  281,  304,   59,  505,
  285,   59,   59,   59,  510,  377,   59,   59,   59,  515,
  382,   59,   59,   59,  321,   59,   59,   59,   59,  304,
   59,  328,   59,  308,  309,   59,   59,  334,   59,  336,
  349,   59,  404,   59,   59,   59,   59,   59,   59,   59,
  256,   59,  349,   59,  416,  256,  418,  419,   59,   59,
   59,  423,   59,   59,   59,  340,  267,  342,  274,   59,
   59,   59,   59,   59,  280,   59,   59,   59,   59,   59,
  442,  356,   59,   59,   59,  382,   59,  288,  289,   61,
  453,   61,  410,  294,  410,  296,   59,  453,  304,  305,
  410,  398,  377,  309,  453,  349,  453,   61,  453,  453,
  453,  453,  453,  453,  410,  388,  425,  323,  410,  416,
   59,  418,  419,  485,  330,   61,  410,  410,   59,  491,
  410,   61,  410,   59,  409,  388,   59,   59,   59,  410,
  502,  503,  410,   59,  350,  442,   59,   59,  423,   59,
  459,  513,  353,  354,   59,   59,   59,   59,  433,   59,
   59,   59,  363,  364,   59,  474,   59,   59,   59,   59,
   59,   59,  378,  410,   61,   61,   61,   61,   61,   61,
  386,  425,  383,  384,  493,  494,   61,   61,  485,   61,
  499,  500,  501,  502,   61,  504,   61,   61,  404,  400,
  401,   61,   61,   61,  513,  502,   61,  413,  414,   61,
  411,   61,   61,   61,   61,  459,  513,  410,  410,  420,
   59,   59,  497,   59,   59,   59,   59,   59,   59,   59,
  474,   59,   59,   59,   59,   59,  511,   59,   59,   59,
  446,  447,   59,   59,   59,   59,   59,   59,  410,  493,
  494,   61,  410,   61,  410,  499,  500,  501,  502,   59,
   59,  460,  461,  462,  463,  464,  410,  466,  467,  513,
  469,  470,  471,  388,  388,  474,  453,  453,  453,  388,
  388,  388,  388,  388,  388,  484,  485,  486,  487,  488,
  489,  410,  410,  492,  453,   61,  495,  503,   59,  453,
   59,   59,   59,   59,   59,  506,  507,  508,  460,  461,
  462,  463,  464,  388,  466,  467,  415,  469,  470,  471,
   59,  410,  474,  410,   59,   59,   59,   59,   59,   59,
  410,  410,  484,  485,  486,  487,  488,  489,  410,   59,
  492,   59,   59,  495,  460,  461,  462,  463,  464,   59,
  466,  467,  387,  469,  470,  471,  410,  410,  474,   59,
  410,  410,   59,   59,   59,  410,  410,   59,  484,  485,
  486,  487,  488,  489,   59,   59,  492,   59,   59,  495,
   59,   59,  458,   44,  410,  415,  453,  453,   61,  453,
  410,  410,   61,  410,  453,   59,  453,   59,  453,   59,
   59,  453,   59,   59,   59,   59,  410,   59,   59,   59,
   59,   59,   59,  453,  453,   59,   59,   59,   59,   59,
   59,   59,   59,   59,   59,  453,   59,   59,  453,   59,
   59,  453,   59,   59,   59,   59,  387,   59,   59,   59,
   59,   59,   59,   59,  453,   59,  453,   59,  453,   59,
  453,  388,   59,   59,   59,   59,   59,   59,  387,   59,
   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,
   59,   59,   59,   59,   59,   59,   59,  123,  123,  277,
  260,  835, 1487, 1485, 1507, 1292,  624,  316, 1414,  123,
  582,   73,  470, 1397, 1399, 1465, 1467,  567,  659, 1468,
  502,  575, 1470, 1412,  365,  376, 1390,  492,  293,  186,
 1245,  126,  445, 1396, 1394,   -1, 1243,   97,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  533,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 516
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,"','",0,0,0,0,0,0,0,0,0,0,0,0,0,0,"';'",0,"'='",0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"ACCEPT_PASSWORD","ACTION","ADMIN","AFTYPE","T_ALLOW","ANTI_NICK_FLOOD",
"ANTI_SPAM_EXIT_MESSAGE_TIME","AUTOCONN","T_BLOCK","BURST_AWAY",
"BURST_TOPICWHO","BYTES","KBYTES","MBYTES","GBYTES","TBYTES","CALLER_ID_WAIT",
"CAN_FLOOD","CAN_IDLE","CHANNEL","CIDR_BITLEN_IPV4","CIDR_BITLEN_IPV6",
"CIPHER_PREFERENCE","CLASS","COMPRESSED","COMPRESSION_LEVEL","CONNECT",
"CONNECTFREQ","CRYPTLINK","DEFAULT_CIPHER_PREFERENCE","DEFAULT_FLOODCOUNT",
"DEFAULT_SPLIT_SERVER_COUNT","DEFAULT_SPLIT_USER_COUNT","DENY","DESCRIPTION",
"DIE","DISABLE_AUTH","DISABLE_FAKE_CHANNELS","DISABLE_HIDDEN",
"DISABLE_LOCAL_CHANNELS","DISABLE_REMOTE_COMMANDS","DOT_IN_IP6_ADDR",
"DOTS_IN_IDENT","DURATION","EGDPOOL_PATH","EMAIL","ENABLE","ENCRYPTED",
"EXCEED_LIMIT","EXEMPT","FAILED_OPER_NOTICE","FAKENAME","IRCD_FLAGS",
"FLATTEN_LINKS","FFAILED_OPERLOG","FKILLLOG","FKLINELOG","FGLINELOG",
"FIOERRLOG","FOPERLOG","FOPERSPYLOG","FUSERLOG","GECOS","GENERAL","GLINE",
"GLINES","GLINE_EXEMPT","GLINE_LOG","GLINE_TIME","GLINE_MIN_CIDR",
"GLINE_MIN_CIDR6","GLOBAL_KILL","IRCD_AUTH","NEED_IDENT","HAVENT_READ_CONF",
"HELP_CHANNEL","HIDDEN","HIDDEN_ADMIN","HIDDEN_NAME","HIDDEN_OPER",
"HIDE_SERVER_IPS","HIDE_SERVERS","HIDE_SPOOF_IPS","HOST","HUB","HUB_MASK",
"IDLETIME","IGNORE_BOGUS_TS","INVISIBLE_ON_CONNECT","IP","KILL",
"KILL_CHASE_TIME_LIMIT","KLINE","KLINE_EXEMPT","KLINE_REASON",
"KLINE_WITH_REASON","KNOCK_DELAY","KNOCK_DELAY_CHANNEL","LAZYLINK","LEAF_MASK",
"LINKS_DELAY","LISTEN","T_LOG","LOGGING","LOG_LEVEL","MAX_ACCEPT","MAX_BANS",
"MAX_CHANS_PER_USER","MAX_GLOBAL","MAX_IDENT","MAX_LOCAL","MAX_NICK_CHANGES",
"MAX_NICK_TIME","MAX_NUMBER","MAX_TARGETS","MESSAGE_LOCALE","MIN_NONWILDCARD",
"MIN_NONWILDCARD_SIMPLE","MODULE","MODULES","NAME","NEED_PASSWORD",
"NETWORK_DESC","NETWORK_NAME","NICK","NICK_CHANGES","NO_CREATE_ON_SPLIT",
"NO_JOIN_ON_SPLIT","NO_OPER_FLOOD","NO_TILDE","NOT","NUMBER","NUMBER_PER_IDENT",
"NUMBER_PER_CIDR","NUMBER_PER_IP","NUMBER_PER_IP_GLOBAL","OPERATOR",
"OPERS_BYPASS_CALLERID","OPER_LOG","OPER_ONLY_UMODES","OPER_PASS_RESV",
"OPER_SPY_T","OPER_UMODES","JOIN_FLOOD_COUNT","JOIN_FLOOD_TIME","PACE_WAIT",
"PACE_WAIT_SIMPLE","PASSWORD","PATH","PING_COOKIE","PING_TIME","PING_WARNING",
"PORT","QSTRING","QUIET_ON_BAN","REASON","REDIRPORT","REDIRSERV","REGEX_T",
"REHASH","TREJECT_HOLD_TIME","REMOTE","REMOTEBAN","RESTRICT_CHANNELS",
"RESTRICTED","RSA_PRIVATE_KEY_FILE","RSA_PUBLIC_KEY_FILE",
"SSL_CERTIFICATE_FILE","RESV","RESV_EXEMPT","SECONDS","MINUTES","HOURS","DAYS",
"WEEKS","SENDQ","SEND_PASSWORD","SERVERHIDE","SERVERINFO","SERVLINK_PATH",
"SERVICES_NAME","IRCD_SID","TKLINE_EXPIRE_NOTICES","T_SHARED","T_CLUSTER",
"T_SNOOP","TYPE","SHORT_MOTD","SILENT","SPOOF","SPOOF_NOTICE",
"STATS_E_DISABLED","STATS_I_OPER_ONLY","STATS_K_OPER_ONLY","STATS_O_OPER_ONLY",
"STATS_P_OPER_ONLY","TBOOL","TMASKED","T_REJECT","TS_MAX_DELTA","TS_WARN_DELTA",
"TWODOTS","T_ALL","T_BOTS","T_SOFTCALLERID","T_CALLERID","T_CCONN",
"T_CCONN_FULL","T_CLIENT_FLOOD","T_DEAF","T_DEBUG","T_DRONE","T_EXTERNAL",
"T_FULL","T_INVISIBLE","T_IPV4","T_IPV6","T_LOCOPS","T_LOGPATH","T_L_CRIT",
"T_L_DEBUG","T_L_ERROR","T_L_INFO","T_L_NOTICE","T_L_TRACE","T_L_WARN",
"T_MAX_CLIENTS","T_NCHANGE","T_OPERWALL","T_REJ","T_SERVNOTICE","T_SKILL",
"T_SPY","T_SSL","T_UMODES","T_UNAUTH","T_UNRESV","T_UNXLINE","T_WALLOP",
"THROTTLE_TIME","TOPICBURST","TRUE_NO_OPER_FLOOD","TKLINE","TXLINE","TRESV",
"UNKLINE","USER","T_USERMODE","USE_EGD","USE_EXCEPT","USE_INVEX","USE_KNOCK",
"USE_LOGGING","USE_WHOIS_ACTUALLY","VHOST","VHOST6","XLINE","WARN",
"WARN_NO_NLINE","WEBIRC",
};
const char * const yyrule[] = {
"$accept : conf",
"conf :",
"conf : conf conf_item",
"conf_item : admin_entry",
"conf_item : logging_entry",
"conf_item : oper_entry",
"conf_item : channel_entry",
"conf_item : class_entry",
"conf_item : listen_entry",
"conf_item : auth_entry",
"conf_item : serverinfo_entry",
"conf_item : serverhide_entry",
"conf_item : resv_entry",
"conf_item : shared_entry",
"conf_item : cluster_entry",
"conf_item : connect_entry",
"conf_item : kill_entry",
"conf_item : deny_entry",
"conf_item : exempt_entry",
"conf_item : general_entry",
"conf_item : gline_entry",
"conf_item : gecos_entry",
"conf_item : modules_entry",
"conf_item : webirc_entry",
"conf_item : error ';'",
"conf_item : error '}'",
"timespec_ :",
"timespec_ : timespec",
"timespec : NUMBER timespec_",
"timespec : NUMBER SECONDS timespec_",
"timespec : NUMBER MINUTES timespec_",
"timespec : NUMBER HOURS timespec_",
"timespec : NUMBER DAYS timespec_",
"timespec : NUMBER WEEKS timespec_",
"sizespec_ :",
"sizespec_ : sizespec",
"sizespec : NUMBER sizespec_",
"sizespec : NUMBER BYTES sizespec_",
"sizespec : NUMBER KBYTES sizespec_",
"sizespec : NUMBER MBYTES sizespec_",
"modules_entry : MODULES '{' modules_items '}' ';'",
"modules_items : modules_items modules_item",
"modules_items : modules_item",
"modules_item : modules_module",
"modules_item : modules_path",
"modules_item : error ';'",
"modules_module : MODULE '=' QSTRING ';'",
"modules_path : PATH '=' QSTRING ';'",
"serverinfo_entry : SERVERINFO '{' serverinfo_items '}' ';'",
"serverinfo_items : serverinfo_items serverinfo_item",
"serverinfo_items : serverinfo_item",
"serverinfo_item : serverinfo_name",
"serverinfo_item : serverinfo_vhost",
"serverinfo_item : serverinfo_hub",
"serverinfo_item : serverinfo_description",
"serverinfo_item : serverinfo_network_name",
"serverinfo_item : serverinfo_network_desc",
"serverinfo_item : serverinfo_services_name",
"serverinfo_item : serverinfo_help_channel",
"serverinfo_item : serverinfo_max_clients",
"serverinfo_item : serverinfo_rsa_private_key_file",
"serverinfo_item : serverinfo_vhost6",
"serverinfo_item : serverinfo_sid",
"serverinfo_item : serverinfo_ssl_certificate_file",
"serverinfo_item : error ';'",
"serverinfo_ssl_certificate_file : SSL_CERTIFICATE_FILE '=' QSTRING ';'",
"serverinfo_rsa_private_key_file : RSA_PRIVATE_KEY_FILE '=' QSTRING ';'",
"serverinfo_name : NAME '=' QSTRING ';'",
"serverinfo_sid : IRCD_SID '=' QSTRING ';'",
"serverinfo_description : DESCRIPTION '=' QSTRING ';'",
"serverinfo_network_name : NETWORK_NAME '=' QSTRING ';'",
"serverinfo_network_desc : NETWORK_DESC '=' QSTRING ';'",
"serverinfo_services_name : SERVICES_NAME '=' QSTRING ';'",
"serverinfo_help_channel : HELP_CHANNEL '=' QSTRING ';'",
"serverinfo_vhost : VHOST '=' QSTRING ';'",
"serverinfo_vhost6 : VHOST6 '=' QSTRING ';'",
"serverinfo_max_clients : T_MAX_CLIENTS '=' NUMBER ';'",
"serverinfo_hub : HUB '=' TBOOL ';'",
"admin_entry : ADMIN '{' admin_items '}' ';'",
"admin_items : admin_items admin_item",
"admin_items : admin_item",
"admin_item : admin_name",
"admin_item : admin_description",
"admin_item : admin_email",
"admin_item : error ';'",
"admin_name : NAME '=' QSTRING ';'",
"admin_email : EMAIL '=' QSTRING ';'",
"admin_description : DESCRIPTION '=' QSTRING ';'",
"logging_entry : LOGGING '{' logging_items '}' ';'",
"logging_items : logging_items logging_item",
"logging_items : logging_item",
"logging_item : logging_path",
"logging_item : logging_oper_log",
"logging_item : logging_log_level",
"logging_item : logging_use_logging",
"logging_item : logging_fuserlog",
"logging_item : logging_foperlog",
"logging_item : logging_fglinelog",
"logging_item : logging_fklinelog",
"logging_item : logging_killlog",
"logging_item : logging_foperspylog",
"logging_item : logging_ioerrlog",
"logging_item : logging_ffailed_operlog",
"logging_item : error ';'",
"logging_path : T_LOGPATH '=' QSTRING ';'",
"logging_oper_log : OPER_LOG '=' QSTRING ';'",
"logging_fuserlog : FUSERLOG '=' QSTRING ';'",
"logging_ffailed_operlog : FFAILED_OPERLOG '=' QSTRING ';'",
"logging_foperlog : FOPERLOG '=' QSTRING ';'",
"logging_foperspylog : FOPERSPYLOG '=' QSTRING ';'",
"logging_fglinelog : FGLINELOG '=' QSTRING ';'",
"logging_fklinelog : FKLINELOG '=' QSTRING ';'",
"logging_ioerrlog : FIOERRLOG '=' QSTRING ';'",
"logging_killlog : FKILLLOG '=' QSTRING ';'",
"logging_log_level : LOG_LEVEL '=' T_L_CRIT ';'",
"logging_log_level : LOG_LEVEL '=' T_L_ERROR ';'",
"logging_log_level : LOG_LEVEL '=' T_L_WARN ';'",
"logging_log_level : LOG_LEVEL '=' T_L_NOTICE ';'",
"logging_log_level : LOG_LEVEL '=' T_L_TRACE ';'",
"logging_log_level : LOG_LEVEL '=' T_L_INFO ';'",
"logging_log_level : LOG_LEVEL '=' T_L_DEBUG ';'",
"logging_use_logging : USE_LOGGING '=' TBOOL ';'",
"$$1 :",
"oper_entry : OPERATOR $$1 oper_name_b '{' oper_items '}' ';'",
"oper_name_b :",
"oper_name_b : oper_name_t",
"oper_items : oper_items oper_item",
"oper_items : oper_item",
"oper_item : oper_name",
"oper_item : oper_user",
"oper_item : oper_password",
"oper_item : oper_hidden_admin",
"oper_item : oper_hidden_oper",
"oper_item : oper_umodes",
"oper_item : oper_class",
"oper_item : oper_global_kill",
"oper_item : oper_remote",
"oper_item : oper_kline",
"oper_item : oper_xline",
"oper_item : oper_unkline",
"oper_item : oper_gline",
"oper_item : oper_nick_changes",
"oper_item : oper_remoteban",
"oper_item : oper_die",
"oper_item : oper_rehash",
"oper_item : oper_admin",
"oper_item : oper_operwall",
"oper_item : oper_snoop",
"oper_item : oper_encrypted",
"oper_item : oper_rsa_public_key_file",
"oper_item : oper_flags",
"oper_item : error ';'",
"oper_name : NAME '=' QSTRING ';'",
"oper_name_t : QSTRING",
"oper_user : USER '=' QSTRING ';'",
"oper_password : PASSWORD '=' QSTRING ';'",
"oper_encrypted : ENCRYPTED '=' TBOOL ';'",
"oper_rsa_public_key_file : RSA_PUBLIC_KEY_FILE '=' QSTRING ';'",
"oper_class : CLASS '=' QSTRING ';'",
"$$2 :",
"oper_umodes : T_UMODES $$2 '=' oper_umodes_items ';'",
"oper_umodes_items : oper_umodes_items ',' oper_umodes_item",
"oper_umodes_items : oper_umodes_item",
"oper_umodes_item : T_BOTS",
"oper_umodes_item : T_CCONN",
"oper_umodes_item : T_CCONN_FULL",
"oper_umodes_item : T_DEAF",
"oper_umodes_item : T_DEBUG",
"oper_umodes_item : T_FULL",
"oper_umodes_item : T_SKILL",
"oper_umodes_item : T_NCHANGE",
"oper_umodes_item : T_REJ",
"oper_umodes_item : T_UNAUTH",
"oper_umodes_item : T_SPY",
"oper_umodes_item : T_EXTERNAL",
"oper_umodes_item : T_OPERWALL",
"oper_umodes_item : T_SERVNOTICE",
"oper_umodes_item : T_INVISIBLE",
"oper_umodes_item : T_WALLOP",
"oper_umodes_item : T_SOFTCALLERID",
"oper_umodes_item : T_CALLERID",
"oper_umodes_item : T_LOCOPS",
"oper_global_kill : GLOBAL_KILL '=' TBOOL ';'",
"oper_remote : REMOTE '=' TBOOL ';'",
"oper_remoteban : REMOTEBAN '=' TBOOL ';'",
"oper_kline : KLINE '=' TBOOL ';'",
"oper_xline : XLINE '=' TBOOL ';'",
"oper_unkline : UNKLINE '=' TBOOL ';'",
"oper_gline : GLINE '=' TBOOL ';'",
"oper_nick_changes : NICK_CHANGES '=' TBOOL ';'",
"oper_die : DIE '=' TBOOL ';'",
"oper_rehash : REHASH '=' TBOOL ';'",
"oper_admin : ADMIN '=' TBOOL ';'",
"oper_hidden_admin : HIDDEN_ADMIN '=' TBOOL ';'",
"oper_hidden_oper : HIDDEN_OPER '=' TBOOL ';'",
"oper_operwall : T_OPERWALL '=' TBOOL ';'",
"oper_snoop : T_SNOOP '=' TBOOL ';'",
"$$3 :",
"oper_flags : IRCD_FLAGS $$3 '=' oper_flags_items ';'",
"oper_flags_items : oper_flags_items ',' oper_flags_item",
"oper_flags_items : oper_flags_item",
"$$4 :",
"oper_flags_item : NOT $$4 oper_flags_item_atom",
"$$5 :",
"oper_flags_item : $$5 oper_flags_item_atom",
"oper_flags_item_atom : GLOBAL_KILL",
"oper_flags_item_atom : REMOTE",
"oper_flags_item_atom : KLINE",
"oper_flags_item_atom : UNKLINE",
"oper_flags_item_atom : XLINE",
"oper_flags_item_atom : GLINE",
"oper_flags_item_atom : DIE",
"oper_flags_item_atom : REHASH",
"oper_flags_item_atom : ADMIN",
"oper_flags_item_atom : HIDDEN_ADMIN",
"oper_flags_item_atom : NICK_CHANGES",
"oper_flags_item_atom : T_OPERWALL",
"oper_flags_item_atom : OPER_SPY_T",
"oper_flags_item_atom : HIDDEN_OPER",
"oper_flags_item_atom : REMOTEBAN",
"oper_flags_item_atom : ENCRYPTED",
"oper_flags_item_atom : T_SNOOP",
"$$6 :",
"webirc_entry : WEBIRC $$6 '{' webirc_items '}' ';'",
"webirc_items : webirc_items webirc_item",
"webirc_items : webirc_item",
"webirc_item : webirc_host",
"webirc_item : webirc_passwd",
"webirc_item : error ';'",
"webirc_host : HOST '=' QSTRING ';'",
"webirc_passwd : PASSWORD '=' QSTRING ';'",
"$$7 :",
"class_entry : CLASS $$7 class_name_b '{' class_items '}' ';'",
"class_name_b :",
"class_name_b : class_name_t",
"class_items : class_items class_item",
"class_items : class_item",
"class_item : class_name",
"class_item : class_cidr_bitlen_ipv4",
"class_item : class_cidr_bitlen_ipv6",
"class_item : class_ping_time",
"class_item : class_ping_warning",
"class_item : class_number_per_cidr",
"class_item : class_number_per_ip",
"class_item : class_connectfreq",
"class_item : class_max_number",
"class_item : class_max_global",
"class_item : class_max_local",
"class_item : class_max_ident",
"class_item : class_sendq",
"class_item : error ';'",
"class_name : NAME '=' QSTRING ';'",
"class_name_t : QSTRING",
"class_ping_time : PING_TIME '=' timespec ';'",
"class_ping_warning : PING_WARNING '=' timespec ';'",
"class_number_per_ip : NUMBER_PER_IP '=' NUMBER ';'",
"class_connectfreq : CONNECTFREQ '=' timespec ';'",
"class_max_number : MAX_NUMBER '=' NUMBER ';'",
"class_max_global : MAX_GLOBAL '=' NUMBER ';'",
"class_max_local : MAX_LOCAL '=' NUMBER ';'",
"class_max_ident : MAX_IDENT '=' NUMBER ';'",
"class_sendq : SENDQ '=' sizespec ';'",
"class_cidr_bitlen_ipv4 : CIDR_BITLEN_IPV4 '=' NUMBER ';'",
"class_cidr_bitlen_ipv6 : CIDR_BITLEN_IPV6 '=' NUMBER ';'",
"class_number_per_cidr : NUMBER_PER_CIDR '=' NUMBER ';'",
"$$8 :",
"listen_entry : LISTEN $$8 '{' listen_items '}' ';'",
"$$9 :",
"listen_flags : IRCD_FLAGS $$9 '=' listen_flags_items ';'",
"listen_flags_items : listen_flags_items ',' listen_flags_item",
"listen_flags_items : listen_flags_item",
"listen_flags_item : T_SSL",
"listen_flags_item : HIDDEN",
"listen_items : listen_items listen_item",
"listen_items : listen_item",
"listen_item : listen_port",
"listen_item : listen_flags",
"listen_item : listen_address",
"listen_item : listen_host",
"listen_item : error ';'",
"$$10 :",
"listen_port : PORT '=' port_items $$10 ';'",
"port_items : port_items ',' port_item",
"port_items : port_item",
"port_item : NUMBER",
"port_item : NUMBER TWODOTS NUMBER",
"listen_address : IP '=' QSTRING ';'",
"listen_host : HOST '=' QSTRING ';'",
"$$11 :",
"auth_entry : IRCD_AUTH $$11 '{' auth_items '}' ';'",
"auth_items : auth_items auth_item",
"auth_items : auth_item",
"auth_item : auth_user",
"auth_item : auth_passwd",
"auth_item : auth_class",
"auth_item : auth_flags",
"auth_item : auth_kline_exempt",
"auth_item : auth_need_ident",
"auth_item : auth_exceed_limit",
"auth_item : auth_no_tilde",
"auth_item : auth_gline_exempt",
"auth_item : auth_spoof",
"auth_item : auth_spoof_notice",
"auth_item : auth_redir_serv",
"auth_item : auth_redir_port",
"auth_item : auth_can_flood",
"auth_item : auth_need_password",
"auth_item : auth_encrypted",
"auth_item : error ';'",
"auth_user : USER '=' QSTRING ';'",
"auth_passwd : PASSWORD '=' QSTRING ';'",
"auth_spoof_notice : SPOOF_NOTICE '=' TBOOL ';'",
"auth_class : CLASS '=' QSTRING ';'",
"auth_encrypted : ENCRYPTED '=' TBOOL ';'",
"$$12 :",
"auth_flags : IRCD_FLAGS $$12 '=' auth_flags_items ';'",
"auth_flags_items : auth_flags_items ',' auth_flags_item",
"auth_flags_items : auth_flags_item",
"$$13 :",
"auth_flags_item : NOT $$13 auth_flags_item_atom",
"$$14 :",
"auth_flags_item : $$14 auth_flags_item_atom",
"auth_flags_item_atom : SPOOF_NOTICE",
"auth_flags_item_atom : EXCEED_LIMIT",
"auth_flags_item_atom : KLINE_EXEMPT",
"auth_flags_item_atom : NEED_IDENT",
"auth_flags_item_atom : CAN_FLOOD",
"auth_flags_item_atom : CAN_IDLE",
"auth_flags_item_atom : NO_TILDE",
"auth_flags_item_atom : GLINE_EXEMPT",
"auth_flags_item_atom : RESV_EXEMPT",
"auth_flags_item_atom : NEED_PASSWORD",
"auth_kline_exempt : KLINE_EXEMPT '=' TBOOL ';'",
"auth_need_ident : NEED_IDENT '=' TBOOL ';'",
"auth_exceed_limit : EXCEED_LIMIT '=' TBOOL ';'",
"auth_can_flood : CAN_FLOOD '=' TBOOL ';'",
"auth_no_tilde : NO_TILDE '=' TBOOL ';'",
"auth_gline_exempt : GLINE_EXEMPT '=' TBOOL ';'",
"auth_spoof : SPOOF '=' QSTRING ';'",
"auth_redir_serv : REDIRSERV '=' QSTRING ';'",
"auth_redir_port : REDIRPORT '=' NUMBER ';'",
"auth_need_password : NEED_PASSWORD '=' TBOOL ';'",
"$$15 :",
"resv_entry : RESV $$15 '{' resv_items '}' ';'",
"resv_items : resv_items resv_item",
"resv_items : resv_item",
"resv_item : resv_creason",
"resv_item : resv_channel",
"resv_item : resv_nick",
"resv_item : error ';'",
"resv_creason : REASON '=' QSTRING ';'",
"resv_channel : CHANNEL '=' QSTRING ';'",
"resv_nick : NICK '=' QSTRING ';'",
"$$16 :",
"shared_entry : T_SHARED $$16 '{' shared_items '}' ';'",
"shared_items : shared_items shared_item",
"shared_items : shared_item",
"shared_item : shared_name",
"shared_item : shared_user",
"shared_item : shared_type",
"shared_item : error ';'",
"shared_name : NAME '=' QSTRING ';'",
"shared_user : USER '=' QSTRING ';'",
"$$17 :",
"shared_type : TYPE $$17 '=' shared_types ';'",
"shared_types : shared_types ',' shared_type_item",
"shared_types : shared_type_item",
"shared_type_item : KLINE",
"shared_type_item : TKLINE",
"shared_type_item : UNKLINE",
"shared_type_item : XLINE",
"shared_type_item : TXLINE",
"shared_type_item : T_UNXLINE",
"shared_type_item : RESV",
"shared_type_item : TRESV",
"shared_type_item : T_UNRESV",
"shared_type_item : T_LOCOPS",
"shared_type_item : T_USERMODE",
"shared_type_item : T_ALL",
"$$18 :",
"cluster_entry : T_CLUSTER $$18 '{' cluster_items '}' ';'",
"cluster_items : cluster_items cluster_item",
"cluster_items : cluster_item",
"cluster_item : cluster_name",
"cluster_item : cluster_type",
"cluster_item : error ';'",
"cluster_name : NAME '=' QSTRING ';'",
"$$19 :",
"cluster_type : TYPE $$19 '=' cluster_types ';'",
"cluster_types : cluster_types ',' cluster_type_item",
"cluster_types : cluster_type_item",
"cluster_type_item : KLINE",
"cluster_type_item : TKLINE",
"cluster_type_item : UNKLINE",
"cluster_type_item : XLINE",
"cluster_type_item : TXLINE",
"cluster_type_item : T_UNXLINE",
"cluster_type_item : RESV",
"cluster_type_item : TRESV",
"cluster_type_item : T_UNRESV",
"cluster_type_item : T_LOCOPS",
"cluster_type_item : T_ALL",
"$$20 :",
"connect_entry : CONNECT $$20 connect_name_b '{' connect_items '}' ';'",
"connect_name_b :",
"connect_name_b : connect_name_t",
"connect_items : connect_items connect_item",
"connect_items : connect_item",
"connect_item : connect_name",
"connect_item : connect_host",
"connect_item : connect_vhost",
"connect_item : connect_send_password",
"connect_item : connect_accept_password",
"connect_item : connect_aftype",
"connect_item : connect_port",
"connect_item : connect_fakename",
"connect_item : connect_flags",
"connect_item : connect_hub_mask",
"connect_item : connect_leaf_mask",
"connect_item : connect_class",
"connect_item : connect_auto",
"connect_item : connect_encrypted",
"connect_item : connect_compressed",
"connect_item : connect_cryptlink",
"connect_item : connect_rsa_public_key_file",
"connect_item : connect_cipher_preference",
"connect_item : connect_topicburst",
"connect_item : error ';'",
"connect_name : NAME '=' QSTRING ';'",
"connect_name_t : QSTRING",
"connect_host : HOST '=' QSTRING ';'",
"connect_vhost : VHOST '=' QSTRING ';'",
"connect_send_password : SEND_PASSWORD '=' QSTRING ';'",
"connect_accept_password : ACCEPT_PASSWORD '=' QSTRING ';'",
"connect_port : PORT '=' NUMBER ';'",
"connect_aftype : AFTYPE '=' T_IPV4 ';'",
"connect_aftype : AFTYPE '=' T_IPV6 ';'",
"connect_fakename : FAKENAME '=' QSTRING ';'",
"$$21 :",
"connect_flags : IRCD_FLAGS $$21 '=' connect_flags_items ';'",
"connect_flags_items : connect_flags_items ',' connect_flags_item",
"connect_flags_items : connect_flags_item",
"$$22 :",
"connect_flags_item : NOT $$22 connect_flags_item_atom",
"$$23 :",
"connect_flags_item : $$23 connect_flags_item_atom",
"connect_flags_item_atom : LAZYLINK",
"connect_flags_item_atom : COMPRESSED",
"connect_flags_item_atom : CRYPTLINK",
"connect_flags_item_atom : AUTOCONN",
"connect_flags_item_atom : BURST_AWAY",
"connect_flags_item_atom : TOPICBURST",
"connect_rsa_public_key_file : RSA_PUBLIC_KEY_FILE '=' QSTRING ';'",
"connect_encrypted : ENCRYPTED '=' TBOOL ';'",
"connect_cryptlink : CRYPTLINK '=' TBOOL ';'",
"connect_compressed : COMPRESSED '=' TBOOL ';'",
"connect_auto : AUTOCONN '=' TBOOL ';'",
"connect_topicburst : TOPICBURST '=' TBOOL ';'",
"connect_hub_mask : HUB_MASK '=' QSTRING ';'",
"connect_leaf_mask : LEAF_MASK '=' QSTRING ';'",
"connect_class : CLASS '=' QSTRING ';'",
"connect_cipher_preference : CIPHER_PREFERENCE '=' QSTRING ';'",
"$$24 :",
"kill_entry : KILL $$24 '{' kill_items '}' ';'",
"$$25 :",
"kill_type : TYPE $$25 '=' kill_type_items ';'",
"kill_type_items : kill_type_items ',' kill_type_item",
"kill_type_items : kill_type_item",
"kill_type_item : REGEX_T",
"kill_items : kill_items kill_item",
"kill_items : kill_item",
"kill_item : kill_user",
"kill_item : kill_reason",
"kill_item : kill_type",
"kill_item : error",
"kill_user : USER '=' QSTRING ';'",
"kill_reason : REASON '=' QSTRING ';'",
"$$26 :",
"deny_entry : DENY $$26 '{' deny_items '}' ';'",
"deny_items : deny_items deny_item",
"deny_items : deny_item",
"deny_item : deny_ip",
"deny_item : deny_reason",
"deny_item : error",
"deny_ip : IP '=' QSTRING ';'",
"deny_reason : REASON '=' QSTRING ';'",
"exempt_entry : EXEMPT '{' exempt_items '}' ';'",
"exempt_items : exempt_items exempt_item",
"exempt_items : exempt_item",
"exempt_item : exempt_ip",
"exempt_item : error",
"exempt_ip : IP '=' QSTRING ';'",
"$$27 :",
"gecos_entry : GECOS $$27 '{' gecos_items '}' ';'",
"$$28 :",
"gecos_flags : TYPE $$28 '=' gecos_flags_items ';'",
"gecos_flags_items : gecos_flags_items ',' gecos_flags_item",
"gecos_flags_items : gecos_flags_item",
"gecos_flags_item : REGEX_T",
"gecos_items : gecos_items gecos_item",
"gecos_items : gecos_item",
"gecos_item : gecos_name",
"gecos_item : gecos_reason",
"gecos_item : gecos_flags",
"gecos_item : error",
"gecos_name : NAME '=' QSTRING ';'",
"gecos_reason : REASON '=' QSTRING ';'",
"general_entry : GENERAL '{' general_items '}' ';'",
"general_items : general_items general_item",
"general_items : general_item",
"general_item : general_hide_spoof_ips",
"general_item : general_ignore_bogus_ts",
"general_item : general_failed_oper_notice",
"general_item : general_anti_nick_flood",
"general_item : general_max_nick_time",
"general_item : general_max_nick_changes",
"general_item : general_max_accept",
"general_item : general_anti_spam_exit_message_time",
"general_item : general_ts_warn_delta",
"general_item : general_ts_max_delta",
"general_item : general_kill_chase_time_limit",
"general_item : general_kline_with_reason",
"general_item : general_kline_reason",
"general_item : general_invisible_on_connect",
"general_item : general_warn_no_nline",
"general_item : general_dots_in_ident",
"general_item : general_stats_o_oper_only",
"general_item : general_stats_k_oper_only",
"general_item : general_pace_wait",
"general_item : general_stats_i_oper_only",
"general_item : general_pace_wait_simple",
"general_item : general_stats_P_oper_only",
"general_item : general_short_motd",
"general_item : general_no_oper_flood",
"general_item : general_true_no_oper_flood",
"general_item : general_oper_pass_resv",
"general_item : general_idletime",
"general_item : general_message_locale",
"general_item : general_oper_only_umodes",
"general_item : general_max_targets",
"general_item : general_use_egd",
"general_item : general_egdpool_path",
"general_item : general_oper_umodes",
"general_item : general_caller_id_wait",
"general_item : general_opers_bypass_callerid",
"general_item : general_default_floodcount",
"general_item : general_min_nonwildcard",
"general_item : general_min_nonwildcard_simple",
"general_item : general_servlink_path",
"general_item : general_disable_remote_commands",
"general_item : general_default_cipher_preference",
"general_item : general_compression_level",
"general_item : general_client_flood",
"general_item : general_throttle_time",
"general_item : general_havent_read_conf",
"general_item : general_dot_in_ip6_addr",
"general_item : general_ping_cookie",
"general_item : general_disable_auth",
"general_item : general_burst_away",
"general_item : general_tkline_expire_notices",
"general_item : general_gline_min_cidr",
"general_item : general_gline_min_cidr6",
"general_item : general_use_whois_actually",
"general_item : general_reject_hold_time",
"general_item : general_stats_e_disabled",
"general_item : error",
"general_gline_min_cidr : GLINE_MIN_CIDR '=' NUMBER ';'",
"general_gline_min_cidr6 : GLINE_MIN_CIDR6 '=' NUMBER ';'",
"general_burst_away : BURST_AWAY '=' TBOOL ';'",
"general_use_whois_actually : USE_WHOIS_ACTUALLY '=' TBOOL ';'",
"general_reject_hold_time : TREJECT_HOLD_TIME '=' timespec ';'",
"general_tkline_expire_notices : TKLINE_EXPIRE_NOTICES '=' TBOOL ';'",
"general_kill_chase_time_limit : KILL_CHASE_TIME_LIMIT '=' NUMBER ';'",
"general_hide_spoof_ips : HIDE_SPOOF_IPS '=' TBOOL ';'",
"general_ignore_bogus_ts : IGNORE_BOGUS_TS '=' TBOOL ';'",
"general_disable_remote_commands : DISABLE_REMOTE_COMMANDS '=' TBOOL ';'",
"general_failed_oper_notice : FAILED_OPER_NOTICE '=' TBOOL ';'",
"general_anti_nick_flood : ANTI_NICK_FLOOD '=' TBOOL ';'",
"general_max_nick_time : MAX_NICK_TIME '=' timespec ';'",
"general_max_nick_changes : MAX_NICK_CHANGES '=' NUMBER ';'",
"general_max_accept : MAX_ACCEPT '=' NUMBER ';'",
"general_anti_spam_exit_message_time : ANTI_SPAM_EXIT_MESSAGE_TIME '=' timespec ';'",
"general_ts_warn_delta : TS_WARN_DELTA '=' timespec ';'",
"general_ts_max_delta : TS_MAX_DELTA '=' timespec ';'",
"general_havent_read_conf : HAVENT_READ_CONF '=' NUMBER ';'",
"general_kline_with_reason : KLINE_WITH_REASON '=' TBOOL ';'",
"general_kline_reason : KLINE_REASON '=' QSTRING ';'",
"general_invisible_on_connect : INVISIBLE_ON_CONNECT '=' TBOOL ';'",
"general_warn_no_nline : WARN_NO_NLINE '=' TBOOL ';'",
"general_stats_e_disabled : STATS_E_DISABLED '=' TBOOL ';'",
"general_stats_o_oper_only : STATS_O_OPER_ONLY '=' TBOOL ';'",
"general_stats_P_oper_only : STATS_P_OPER_ONLY '=' TBOOL ';'",
"general_stats_k_oper_only : STATS_K_OPER_ONLY '=' TBOOL ';'",
"general_stats_k_oper_only : STATS_K_OPER_ONLY '=' TMASKED ';'",
"general_stats_i_oper_only : STATS_I_OPER_ONLY '=' TBOOL ';'",
"general_stats_i_oper_only : STATS_I_OPER_ONLY '=' TMASKED ';'",
"general_pace_wait : PACE_WAIT '=' timespec ';'",
"general_caller_id_wait : CALLER_ID_WAIT '=' timespec ';'",
"general_opers_bypass_callerid : OPERS_BYPASS_CALLERID '=' TBOOL ';'",
"general_pace_wait_simple : PACE_WAIT_SIMPLE '=' timespec ';'",
"general_short_motd : SHORT_MOTD '=' TBOOL ';'",
"general_no_oper_flood : NO_OPER_FLOOD '=' TBOOL ';'",
"general_true_no_oper_flood : TRUE_NO_OPER_FLOOD '=' TBOOL ';'",
"general_oper_pass_resv : OPER_PASS_RESV '=' TBOOL ';'",
"general_message_locale : MESSAGE_LOCALE '=' QSTRING ';'",
"general_idletime : IDLETIME '=' timespec ';'",
"general_dots_in_ident : DOTS_IN_IDENT '=' NUMBER ';'",
"general_max_targets : MAX_TARGETS '=' NUMBER ';'",
"general_servlink_path : SERVLINK_PATH '=' QSTRING ';'",
"general_default_cipher_preference : DEFAULT_CIPHER_PREFERENCE '=' QSTRING ';'",
"general_compression_level : COMPRESSION_LEVEL '=' NUMBER ';'",
"general_use_egd : USE_EGD '=' TBOOL ';'",
"general_egdpool_path : EGDPOOL_PATH '=' QSTRING ';'",
"general_ping_cookie : PING_COOKIE '=' TBOOL ';'",
"general_disable_auth : DISABLE_AUTH '=' TBOOL ';'",
"general_throttle_time : THROTTLE_TIME '=' timespec ';'",
"$$29 :",
"general_oper_umodes : OPER_UMODES $$29 '=' umode_oitems ';'",
"umode_oitems : umode_oitems ',' umode_oitem",
"umode_oitems : umode_oitem",
"umode_oitem : T_BOTS",
"umode_oitem : T_CCONN",
"umode_oitem : T_CCONN_FULL",
"umode_oitem : T_DEAF",
"umode_oitem : T_DEBUG",
"umode_oitem : T_FULL",
"umode_oitem : T_SKILL",
"umode_oitem : T_NCHANGE",
"umode_oitem : T_REJ",
"umode_oitem : T_UNAUTH",
"umode_oitem : T_SPY",
"umode_oitem : T_EXTERNAL",
"umode_oitem : T_OPERWALL",
"umode_oitem : T_SERVNOTICE",
"umode_oitem : T_INVISIBLE",
"umode_oitem : T_WALLOP",
"umode_oitem : T_SOFTCALLERID",
"umode_oitem : T_CALLERID",
"umode_oitem : T_LOCOPS",
"$$30 :",
"general_oper_only_umodes : OPER_ONLY_UMODES $$30 '=' umode_items ';'",
"umode_items : umode_items ',' umode_item",
"umode_items : umode_item",
"umode_item : T_BOTS",
"umode_item : T_CCONN",
"umode_item : T_CCONN_FULL",
"umode_item : T_DEAF",
"umode_item : T_DEBUG",
"umode_item : T_FULL",
"umode_item : T_SKILL",
"umode_item : T_NCHANGE",
"umode_item : T_REJ",
"umode_item : T_UNAUTH",
"umode_item : T_SPY",
"umode_item : T_EXTERNAL",
"umode_item : T_OPERWALL",
"umode_item : T_SERVNOTICE",
"umode_item : T_INVISIBLE",
"umode_item : T_WALLOP",
"umode_item : T_SOFTCALLERID",
"umode_item : T_CALLERID",
"umode_item : T_LOCOPS",
"general_min_nonwildcard : MIN_NONWILDCARD '=' NUMBER ';'",
"general_min_nonwildcard_simple : MIN_NONWILDCARD_SIMPLE '=' NUMBER ';'",
"general_default_floodcount : DEFAULT_FLOODCOUNT '=' NUMBER ';'",
"general_client_flood : T_CLIENT_FLOOD '=' sizespec ';'",
"general_dot_in_ip6_addr : DOT_IN_IP6_ADDR '=' TBOOL ';'",
"$$31 :",
"gline_entry : GLINES $$31 '{' gline_items '}' ';'",
"gline_items : gline_items gline_item",
"gline_items : gline_item",
"gline_item : gline_enable",
"gline_item : gline_duration",
"gline_item : gline_logging",
"gline_item : gline_user",
"gline_item : gline_server",
"gline_item : gline_action",
"gline_item : error",
"gline_enable : ENABLE '=' TBOOL ';'",
"gline_duration : DURATION '=' timespec ';'",
"$$32 :",
"gline_logging : LOGGING $$32 '=' gline_logging_types ';'",
"gline_logging_types : gline_logging_types ',' gline_logging_type_item",
"gline_logging_types : gline_logging_type_item",
"gline_logging_type_item : T_REJECT",
"gline_logging_type_item : T_BLOCK",
"gline_user : USER '=' QSTRING ';'",
"gline_server : NAME '=' QSTRING ';'",
"$$33 :",
"gline_action : ACTION $$33 '=' gdeny_types ';'",
"gdeny_types : gdeny_types ',' gdeny_type_item",
"gdeny_types : gdeny_type_item",
"gdeny_type_item : T_REJECT",
"gdeny_type_item : T_BLOCK",
"channel_entry : CHANNEL '{' channel_items '}' ';'",
"channel_items : channel_items channel_item",
"channel_items : channel_item",
"channel_item : channel_disable_local_channels",
"channel_item : channel_use_except",
"channel_item : channel_use_invex",
"channel_item : channel_use_knock",
"channel_item : channel_max_bans",
"channel_item : channel_knock_delay",
"channel_item : channel_knock_delay_channel",
"channel_item : channel_max_chans_per_user",
"channel_item : channel_quiet_on_ban",
"channel_item : channel_default_split_user_count",
"channel_item : channel_default_split_server_count",
"channel_item : channel_no_create_on_split",
"channel_item : channel_restrict_channels",
"channel_item : channel_no_join_on_split",
"channel_item : channel_burst_topicwho",
"channel_item : channel_jflood_count",
"channel_item : channel_jflood_time",
"channel_item : channel_disable_fake_channels",
"channel_item : error",
"channel_disable_fake_channels : DISABLE_FAKE_CHANNELS '=' TBOOL ';'",
"channel_restrict_channels : RESTRICT_CHANNELS '=' TBOOL ';'",
"channel_disable_local_channels : DISABLE_LOCAL_CHANNELS '=' TBOOL ';'",
"channel_use_except : USE_EXCEPT '=' TBOOL ';'",
"channel_use_invex : USE_INVEX '=' TBOOL ';'",
"channel_use_knock : USE_KNOCK '=' TBOOL ';'",
"channel_knock_delay : KNOCK_DELAY '=' timespec ';'",
"channel_knock_delay_channel : KNOCK_DELAY_CHANNEL '=' timespec ';'",
"channel_max_chans_per_user : MAX_CHANS_PER_USER '=' NUMBER ';'",
"channel_quiet_on_ban : QUIET_ON_BAN '=' TBOOL ';'",
"channel_max_bans : MAX_BANS '=' NUMBER ';'",
"channel_default_split_user_count : DEFAULT_SPLIT_USER_COUNT '=' NUMBER ';'",
"channel_default_split_server_count : DEFAULT_SPLIT_SERVER_COUNT '=' NUMBER ';'",
"channel_no_create_on_split : NO_CREATE_ON_SPLIT '=' TBOOL ';'",
"channel_no_join_on_split : NO_JOIN_ON_SPLIT '=' TBOOL ';'",
"channel_burst_topicwho : BURST_TOPICWHO '=' TBOOL ';'",
"channel_jflood_count : JOIN_FLOOD_COUNT '=' NUMBER ';'",
"channel_jflood_time : JOIN_FLOOD_TIME '=' timespec ';'",
"serverhide_entry : SERVERHIDE '{' serverhide_items '}' ';'",
"serverhide_items : serverhide_items serverhide_item",
"serverhide_items : serverhide_item",
"serverhide_item : serverhide_flatten_links",
"serverhide_item : serverhide_hide_servers",
"serverhide_item : serverhide_links_delay",
"serverhide_item : serverhide_disable_hidden",
"serverhide_item : serverhide_hidden",
"serverhide_item : serverhide_hidden_name",
"serverhide_item : serverhide_hide_server_ips",
"serverhide_item : error",
"serverhide_flatten_links : FLATTEN_LINKS '=' TBOOL ';'",
"serverhide_hide_servers : HIDE_SERVERS '=' TBOOL ';'",
"serverhide_hidden_name : HIDDEN_NAME '=' QSTRING ';'",
"serverhide_links_delay : LINKS_DELAY '=' timespec ';'",
"serverhide_hidden : HIDDEN '=' TBOOL ';'",
"serverhide_disable_hidden : DISABLE_HIDDEN '=' TBOOL ';'",
"serverhide_hide_server_ips : HIDE_SERVER_IPS '=' TBOOL ';'",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 26:
#line 430 "ircd_parser.y"
{ yyval.number = 0; }
break;
case 28:
#line 432 "ircd_parser.y"
{
			yyval.number = yyvsp[-1].number + yyvsp[0].number;
		}
break;
case 29:
#line 436 "ircd_parser.y"
{
			yyval.number = yyvsp[-2].number + yyvsp[0].number;
		}
break;
case 30:
#line 440 "ircd_parser.y"
{
			yyval.number = yyvsp[-2].number * 60 + yyvsp[0].number;
		}
break;
case 31:
#line 444 "ircd_parser.y"
{
			yyval.number = yyvsp[-2].number * 60 * 60 + yyvsp[0].number;
		}
break;
case 32:
#line 448 "ircd_parser.y"
{
			yyval.number = yyvsp[-2].number * 60 * 60 * 24 + yyvsp[0].number;
		}
break;
case 33:
#line 452 "ircd_parser.y"
{
			yyval.number = yyvsp[-2].number * 60 * 60 * 24 * 7 + yyvsp[0].number;
		}
break;
case 34:
#line 457 "ircd_parser.y"
{ yyval.number = 0; }
break;
case 36:
#line 458 "ircd_parser.y"
{ yyval.number = yyvsp[-1].number + yyvsp[0].number; }
break;
case 37:
#line 459 "ircd_parser.y"
{ yyval.number = yyvsp[-2].number + yyvsp[0].number; }
break;
case 38:
#line 460 "ircd_parser.y"
{ yyval.number = yyvsp[-2].number * 1024 + yyvsp[0].number; }
break;
case 39:
#line 461 "ircd_parser.y"
{ yyval.number = yyvsp[-2].number * 1024 * 1024 + yyvsp[0].number; }
break;
case 46:
#line 475 "ircd_parser.y"
{
#ifndef STATIC_MODULES /* NOOP in the static case */
  if (ypass == 2)
  {
    char *m_bn;

    m_bn = basename(yylval.string);

    /* I suppose we should just ignore it if it is already loaded(since
     * otherwise we would flood the opers on rehash) -A1kmm.
     */
    add_conf_module(yylval.string);
  }
#endif
}
break;
case 47:
#line 492 "ircd_parser.y"
{
#ifndef STATIC_MODULES
  if (ypass == 2)
    mod_add_path(yylval.string);
#endif
}
break;
case 65:
#line 517 "ircd_parser.y"
{
#ifdef HAVE_LIBCRYPTO
  if (ypass == 2 && ServerInfo.ctx) 
  {
    if (!ServerInfo.rsa_private_key_file)
    {
      yyerror("No rsa_private_key_file specified, SSL disabled");
      break;
    }

    if (SSL_CTX_use_certificate_file(ServerInfo.ctx,
      yylval.string, SSL_FILETYPE_PEM) <= 0)
    {
      yyerror(ERR_lib_error_string(ERR_get_error()));
      break;
    }

    if (SSL_CTX_use_PrivateKey_file(ServerInfo.ctx,
      ServerInfo.rsa_private_key_file, SSL_FILETYPE_PEM) <= 0)
    {
      yyerror(ERR_lib_error_string(ERR_get_error()));
      break;
    }

    if (!SSL_CTX_check_private_key(ServerInfo.ctx))
    {
      yyerror("RSA private key does not match the SSL certificate public key!");
      break;
    }
  }
#endif
}
break;
case 66:
#line 551 "ircd_parser.y"
{
#ifdef HAVE_LIBCRYPTO
  if (ypass == 1)
  {
    BIO *file;

    if (ServerInfo.rsa_private_key)
    {
      RSA_free(ServerInfo.rsa_private_key);
      ServerInfo.rsa_private_key = NULL;
    }

    if (ServerInfo.rsa_private_key_file)
    {
      MyFree(ServerInfo.rsa_private_key_file);
      ServerInfo.rsa_private_key_file = NULL;
    }

    DupString(ServerInfo.rsa_private_key_file, yylval.string);

    if ((file = BIO_new_file(yylval.string, "r")) == NULL)
    {
      yyerror("File open failed, ignoring");
      break;
    }

    ServerInfo.rsa_private_key = (RSA *)PEM_read_bio_RSAPrivateKey(file, NULL,
      0, NULL);

    BIO_set_close(file, BIO_CLOSE);
    BIO_free(file);

    if (ServerInfo.rsa_private_key == NULL)
    {
      yyerror("Couldn't extract key, ignoring");
      break;
    }

    if (!RSA_check_key(ServerInfo.rsa_private_key))
    {
      RSA_free(ServerInfo.rsa_private_key);
      ServerInfo.rsa_private_key = NULL;

      yyerror("Invalid key, ignoring");
      break;
    }

    /* require 2048 bit (256 byte) key */
    if (RSA_size(ServerInfo.rsa_private_key) != 256)
    {
      RSA_free(ServerInfo.rsa_private_key);
      ServerInfo.rsa_private_key = NULL;

      yyerror("Not a 2048 bit key, ignoring");
    }
  }
#endif
}
break;
case 67:
#line 611 "ircd_parser.y"
{
  /* this isn't rehashable */
  if (ypass == 2)
  {
    if (ServerInfo.name == NULL)
    {
      /* the ircd will exit() in main() if we dont set one */
      if (strlen(yylval.string) <= HOSTLEN)
        DupString(ServerInfo.name, yylval.string);
    }
  }
}
break;
case 68:
#line 625 "ircd_parser.y"
{
  /* this isn't rehashable */
  if (ypass == 2 && !ServerInfo.sid)
  {
    if (valid_sid(yylval.string))
      DupString(ServerInfo.sid, yylval.string);
    else
    {
      ilog(L_ERROR, "Ignoring config file entry SID -- invalid SID. Aborting.");
      exit(0);
    }
  }
}
break;
case 69:
#line 640 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ServerInfo.description);
    DupString(ServerInfo.description,yylval.string);
  }
}
break;
case 70:
#line 649 "ircd_parser.y"
{
  if (ypass == 2)
  {
    char *p;

    if ((p = strchr(yylval.string, ' ')) != NULL)
      p = '\0';

    MyFree(ServerInfo.network_name);
    DupString(ServerInfo.network_name, yylval.string);
  }
}
break;
case 71:
#line 663 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ServerInfo.network_desc);
    DupString(ServerInfo.network_desc, yylval.string);
  }
}
break;
case 72:
#line 672 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ServerInfo.services_name);
    DupString(ServerInfo.services_name, yylval.string);
  }
}
break;
case 73:
#line 681 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ServerInfo.help_channel);
    DupString(ServerInfo.help_channel, yylval.string);
  }
}
break;
case 74:
#line 690 "ircd_parser.y"
{
  if (ypass == 2 && *yylval.string != '*')
  {
    struct addrinfo hints, *res;

    memset(&hints, 0, sizeof(hints));

    hints.ai_family   = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags    = AI_PASSIVE | AI_NUMERICHOST;

    if (irc_getaddrinfo(yylval.string, NULL, &hints, &res))
      ilog(L_ERROR, "Invalid netmask for server vhost(%s)", yylval.string);
    else
    {
      assert(res != NULL);

      memcpy(&ServerInfo.ip, res->ai_addr, res->ai_addrlen);
      ServerInfo.ip.ss.ss_family = res->ai_family;
      ServerInfo.ip.ss_len = res->ai_addrlen;
      irc_freeaddrinfo(res);

      ServerInfo.specific_ipv4_vhost = 1;
    }
  }
}
break;
case 75:
#line 718 "ircd_parser.y"
{
#ifdef IPV6
  if (ypass == 2 && *yylval.string != '*')
  {
    struct addrinfo hints, *res;

    memset(&hints, 0, sizeof(hints));

    hints.ai_family   = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags    = AI_PASSIVE | AI_NUMERICHOST;

    if (irc_getaddrinfo(yylval.string, NULL, &hints, &res))
      ilog(L_ERROR, "Invalid netmask for server vhost6(%s)", yylval.string);
    else
    {
      assert(res != NULL);

      memcpy(&ServerInfo.ip6, res->ai_addr, res->ai_addrlen);
      ServerInfo.ip6.ss.ss_family = res->ai_family;
      ServerInfo.ip6.ss_len = res->ai_addrlen;
      irc_freeaddrinfo(res);

      ServerInfo.specific_ipv6_vhost = 1;
    }
  }
#endif
}
break;
case 76:
#line 748 "ircd_parser.y"
{
  if (ypass == 2)
  {
    recalc_fdlimit(NULL);

    if (yyvsp[-1].number < MAXCLIENTS_MIN)
    {
      char buf[IRCD_BUFSIZE];
      ircsprintf(buf, "MAXCLIENTS too low, setting to %d", MAXCLIENTS_MIN);
      yyerror(buf);
    }
    else if (yyvsp[-1].number > MAXCLIENTS_MAX)
    {
      char buf[IRCD_BUFSIZE];
      ircsprintf(buf, "MAXCLIENTS too high, setting to %d", MAXCLIENTS_MAX);
      yyerror(buf);
    }
    else
      ServerInfo.max_clients = yyvsp[-1].number;
  }
}
break;
case 77:
#line 771 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
    {
      /* Don't become a hub if we have a lazylink active. */
      if (!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
      {
        sendto_realops_flags(UMODE_ALL, L_ALL,
                             "Ignoring config file line hub=yes; "
                             "due to active LazyLink (%s)", uplink->name);
      }
      else
      {
        ServerInfo.hub = 1;
        uplink = NULL;
        delete_capability("HUB");
        add_capability("HUB", CAP_HUB, 1);
      }
    }
    else if (ServerInfo.hub)
    {
      dlink_node *ptr = NULL;

      ServerInfo.hub = 0;
      delete_capability("HUB");

      /* Don't become a leaf if we have a lazylink active. */
      DLINK_FOREACH(ptr, serv_list.head)
      {
        const struct Client *acptr = ptr->data;
        if (MyConnect(acptr) && IsCapable(acptr, CAP_LL))
        {
          sendto_realops_flags(UMODE_ALL, L_ALL,
                               "Ignoring config file line hub=no; "
                               "due to active LazyLink (%s)",
                               acptr->name);
          add_capability("HUB", CAP_HUB, 1);
          ServerInfo.hub = 1;
          break;
        }
      }
    }
  }
}
break;
case 85:
#line 827 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(AdminInfo.name);
    DupString(AdminInfo.name, yylval.string);
  }
}
break;
case 86:
#line 836 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(AdminInfo.email);
    DupString(AdminInfo.email, yylval.string);
  }
}
break;
case 87:
#line 845 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(AdminInfo.description);
    DupString(AdminInfo.description, yylval.string);
  }
}
break;
case 104:
#line 872 "ircd_parser.y"
{
                        }
break;
case 105:
#line 876 "ircd_parser.y"
{
                        }
break;
case 106:
#line 880 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.userlog, yylval.string,
            sizeof(ConfigLoggingEntry.userlog));
}
break;
case 107:
#line 887 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.failed_operlog, yylval.string,
            sizeof(ConfigLoggingEntry.failed_operlog));
}
break;
case 108:
#line 894 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.operlog, yylval.string,
            sizeof(ConfigLoggingEntry.operlog));
}
break;
case 109:
#line 901 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.operspylog, yylval.string,
            sizeof(ConfigLoggingEntry.operspylog));
}
break;
case 110:
#line 908 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.glinelog, yylval.string,
            sizeof(ConfigLoggingEntry.glinelog));
}
break;
case 111:
#line 915 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.klinelog, yylval.string,
            sizeof(ConfigLoggingEntry.klinelog));
}
break;
case 112:
#line 922 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.ioerrlog, yylval.string,
            sizeof(ConfigLoggingEntry.ioerrlog));
}
break;
case 113:
#line 929 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(ConfigLoggingEntry.killlog, yylval.string,
            sizeof(ConfigLoggingEntry.killlog));
}
break;
case 114:
#line 936 "ircd_parser.y"
{ 
  if (ypass == 2)
    set_log_level(L_CRIT);
}
break;
case 115:
#line 940 "ircd_parser.y"
{
  if (ypass == 2)
    set_log_level(L_ERROR);
}
break;
case 116:
#line 944 "ircd_parser.y"
{
  if (ypass == 2)
    set_log_level(L_WARN);
}
break;
case 117:
#line 948 "ircd_parser.y"
{
  if (ypass == 2)
    set_log_level(L_NOTICE);
}
break;
case 118:
#line 952 "ircd_parser.y"
{
  if (ypass == 2)
    set_log_level(L_TRACE);
}
break;
case 119:
#line 956 "ircd_parser.y"
{
  if (ypass == 2)
    set_log_level(L_INFO);
}
break;
case 120:
#line 960 "ircd_parser.y"
{
  if (ypass == 2)
    set_log_level(L_DEBUG);
}
break;
case 121:
#line 966 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigLoggingEntry.use_logging = yylval.number;
}
break;
case 122:
#line 975 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(OPER_TYPE);
    yy_aconf = map_to_conf(yy_conf);
    SetConfEncrypted(yy_aconf); /* Yes, the default is encrypted */
  }
  else
  {
    MyFree(class_name);
    class_name = NULL;
  }
}
break;
case 123:
#line 988 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_tmp;
    dlink_node *ptr;
    dlink_node *next_ptr;

    conf_add_class_to_conf(yy_conf, class_name);

    /* Now, make sure there is a copy of the "base" given oper
     * block in each of the collected copies
     */

    DLINK_FOREACH_SAFE(ptr, next_ptr, col_conf_list.head)
    {
      struct AccessItem *new_aconf;
      struct ConfItem *new_conf;
      yy_tmp = ptr->data;

      new_conf = make_conf_item(OPER_TYPE);
      new_aconf = (struct AccessItem *)map_to_conf(new_conf);

      new_aconf->flags = yy_aconf->flags;

      if (yy_conf->name != NULL)
        DupString(new_conf->name, yy_conf->name);
      if (yy_tmp->user != NULL)
	DupString(new_aconf->user, yy_tmp->user);
      else
	DupString(new_aconf->user, "*");
      if (yy_tmp->host != NULL)
	DupString(new_aconf->host, yy_tmp->host);
      else
	DupString(new_aconf->host, "*");
      conf_add_class_to_conf(new_conf, class_name);
      if (yy_aconf->passwd != NULL)
        DupString(new_aconf->passwd, yy_aconf->passwd);

      new_aconf->port = yy_aconf->port;
#ifdef HAVE_LIBCRYPTO
      if (yy_aconf->rsa_public_key_file != NULL)
      {
        BIO *file;

        DupString(new_aconf->rsa_public_key_file,
		  yy_aconf->rsa_public_key_file);

        file = BIO_new_file(yy_aconf->rsa_public_key_file, "r");
        new_aconf->rsa_public_key = (RSA *)PEM_read_bio_RSA_PUBKEY(file, 
							   NULL, 0, NULL);
        BIO_set_close(file, BIO_CLOSE);
        BIO_free(file);
      }
#endif

#ifdef HAVE_LIBCRYPTO
      if (yy_tmp->name && (yy_tmp->passwd || yy_aconf->rsa_public_key)
	  && yy_tmp->host)
#else
      if (yy_tmp->name && yy_tmp->passwd && yy_tmp->host)
#endif
      {
        conf_add_class_to_conf(new_conf, class_name);
	if (yy_tmp->name != NULL)
	  DupString(new_conf->name, yy_tmp->name);
      }

      dlinkDelete(&yy_tmp->node, &col_conf_list);
      free_collect_item(yy_tmp);
    }

    yy_conf = NULL;
    yy_aconf = NULL;


    MyFree(class_name);
    class_name = NULL;
  }
}
break;
case 152:
#line 1080 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (strlen(yylval.string) > OPERNICKLEN)
      yylval.string[OPERNICKLEN] = '\0';

    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 153:
#line 1092 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (strlen(yylval.string) > OPERNICKLEN)
      yylval.string[OPERNICKLEN] = '\0';

    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 154:
#line 1104 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct split_nuh_item nuh;

    nuh.nuhmask  = yylval.string;
    nuh.nickptr  = NULL;
    nuh.userptr  = userbuf;
    nuh.hostptr  = hostbuf;

    nuh.nicksize = 0;
    nuh.usersize = sizeof(userbuf);
    nuh.hostsize = sizeof(hostbuf);

    split_nuh(&nuh);

    if (yy_aconf->user == NULL)
    {
      DupString(yy_aconf->user, userbuf);
      DupString(yy_aconf->host, hostbuf);
    }
    else
    {
      struct CollectItem *yy_tmp = MyMalloc(sizeof(struct CollectItem));

      DupString(yy_tmp->user, userbuf);
      DupString(yy_tmp->host, hostbuf);

      dlinkAdd(yy_tmp, &yy_tmp->node, &col_conf_list);
    }
  }
}
break;
case 155:
#line 1138 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yy_aconf->passwd != NULL)
      memset(yy_aconf->passwd, 0, strlen(yy_aconf->passwd));

    MyFree(yy_aconf->passwd);
    DupString(yy_aconf->passwd, yylval.string);
  }
}
break;
case 156:
#line 1150 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      SetConfEncrypted(yy_aconf);
    else
      ClearConfEncrypted(yy_aconf);
  }
}
break;
case 157:
#line 1161 "ircd_parser.y"
{
#ifdef HAVE_LIBCRYPTO
  if (ypass == 2)
  {
    BIO *file;

    if (yy_aconf->rsa_public_key != NULL)
    {
      RSA_free(yy_aconf->rsa_public_key);
      yy_aconf->rsa_public_key = NULL;
    }

    if (yy_aconf->rsa_public_key_file != NULL)
    {
      MyFree(yy_aconf->rsa_public_key_file);
      yy_aconf->rsa_public_key_file = NULL;
    }

    DupString(yy_aconf->rsa_public_key_file, yylval.string);
    file = BIO_new_file(yylval.string, "r");

    if (file == NULL)
    {
      yyerror("Ignoring rsa_public_key_file -- file doesn't exist");
      break;
    }

    yy_aconf->rsa_public_key = (RSA *)PEM_read_bio_RSA_PUBKEY(file, NULL, 0, NULL);

    if (yy_aconf->rsa_public_key == NULL)
    {
      yyerror("Ignoring rsa_public_key_file -- Key invalid; check key syntax.");
      break;
    }

    BIO_set_close(file, BIO_CLOSE);
    BIO_free(file);
  }
#endif /* HAVE_LIBCRYPTO */
}
break;
case 158:
#line 1203 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(class_name);
    DupString(class_name, yylval.string);
  }
}
break;
case 159:
#line 1212 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes = 0;
}
break;
case 163:
#line 1219 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_BOTS;
}
break;
case 164:
#line 1223 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_CCONN;
}
break;
case 165:
#line 1227 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_CCONN_FULL;
}
break;
case 166:
#line 1231 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_DEAF;
}
break;
case 167:
#line 1235 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_DEBUG;
}
break;
case 168:
#line 1239 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_FULL;
}
break;
case 169:
#line 1243 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_SKILL;
}
break;
case 170:
#line 1247 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_NCHANGE;
}
break;
case 171:
#line 1251 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_REJ;
}
break;
case 172:
#line 1255 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_UNAUTH;
}
break;
case 173:
#line 1259 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_SPY;
}
break;
case 174:
#line 1263 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_EXTERNAL;
}
break;
case 175:
#line 1267 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_OPERWALL;
}
break;
case 176:
#line 1271 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_SERVNOTICE;
}
break;
case 177:
#line 1275 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_INVISIBLE;
}
break;
case 178:
#line 1279 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_WALLOP;
}
break;
case 179:
#line 1283 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_SOFTCALLERID;
}
break;
case 180:
#line 1287 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_CALLERID;
}
break;
case 181:
#line 1291 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->modes |= UMODE_LOCOPS;
}
break;
case 182:
#line 1297 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_GLOBAL_KILL;
    else
      yy_aconf->port &= ~OPER_FLAG_GLOBAL_KILL;
  }
}
break;
case 183:
#line 1308 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_REMOTE;
    else
      yy_aconf->port &= ~OPER_FLAG_REMOTE; 
  }
}
break;
case 184:
#line 1319 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_REMOTEBAN;
    else
      yy_aconf->port &= ~OPER_FLAG_REMOTEBAN;
  }
}
break;
case 185:
#line 1330 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_K;
    else
      yy_aconf->port &= ~OPER_FLAG_K;
  }
}
break;
case 186:
#line 1341 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_X;
    else
      yy_aconf->port &= ~OPER_FLAG_X;
  }
}
break;
case 187:
#line 1352 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_UNKLINE;
    else
      yy_aconf->port &= ~OPER_FLAG_UNKLINE; 
  }
}
break;
case 188:
#line 1363 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_GLINE;
    else
      yy_aconf->port &= ~OPER_FLAG_GLINE;
  }
}
break;
case 189:
#line 1374 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_N;
    else
      yy_aconf->port &= ~OPER_FLAG_N;
  }
}
break;
case 190:
#line 1385 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_DIE;
    else
      yy_aconf->port &= ~OPER_FLAG_DIE;
  }
}
break;
case 191:
#line 1396 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_REHASH;
    else
      yy_aconf->port &= ~OPER_FLAG_REHASH;
  }
}
break;
case 192:
#line 1407 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_ADMIN;
    else
      yy_aconf->port &= ~OPER_FLAG_ADMIN;
  }
}
break;
case 193:
#line 1418 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_HIDDEN_ADMIN;
    else
      yy_aconf->port &= ~OPER_FLAG_HIDDEN_ADMIN;
  }
}
break;
case 194:
#line 1429 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_HIDDEN_OPER;
    else
      yy_aconf->port &= ~OPER_FLAG_HIDDEN_OPER;
  }
}
break;
case 195:
#line 1440 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_OPERWALL;
    else
      yy_aconf->port &= ~OPER_FLAG_OPERWALL;
  }
}
break;
case 196:
#line 1451 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->port |= OPER_FLAG_SNOOP;
    else
      yy_aconf->port &= ~OPER_FLAG_SNOOP;
  }
}
break;
case 197:
#line 1462 "ircd_parser.y"
{
}
break;
case 201:
#line 1466 "ircd_parser.y"
{ not_atom = 1; }
break;
case 203:
#line 1467 "ircd_parser.y"
{ not_atom = 0; }
break;
case 205:
#line 1470 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom)yy_aconf->port &= ~OPER_FLAG_GLOBAL_KILL;
    else yy_aconf->port |= OPER_FLAG_GLOBAL_KILL;
  }
}
break;
case 206:
#line 1477 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_REMOTE;
    else yy_aconf->port |= OPER_FLAG_REMOTE;
  }
}
break;
case 207:
#line 1484 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_K;
    else yy_aconf->port |= OPER_FLAG_K;
  }
}
break;
case 208:
#line 1491 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_UNKLINE;
    else yy_aconf->port |= OPER_FLAG_UNKLINE;
  } 
}
break;
case 209:
#line 1498 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_X;
    else yy_aconf->port |= OPER_FLAG_X;
  }
}
break;
case 210:
#line 1505 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_GLINE;
    else yy_aconf->port |= OPER_FLAG_GLINE;
  }
}
break;
case 211:
#line 1512 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_DIE;
    else yy_aconf->port |= OPER_FLAG_DIE;
  }
}
break;
case 212:
#line 1519 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_REHASH;
    else yy_aconf->port |= OPER_FLAG_REHASH;
  }
}
break;
case 213:
#line 1526 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_ADMIN;
    else yy_aconf->port |= OPER_FLAG_ADMIN;
  }
}
break;
case 214:
#line 1533 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_HIDDEN_ADMIN;
    else yy_aconf->port |= OPER_FLAG_HIDDEN_ADMIN;
  }
}
break;
case 215:
#line 1540 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_N;
    else yy_aconf->port |= OPER_FLAG_N;
  }
}
break;
case 216:
#line 1547 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_OPERWALL;
    else yy_aconf->port |= OPER_FLAG_OPERWALL;
  }
}
break;
case 217:
#line 1554 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_OPER_SPY;
    else yy_aconf->port |= OPER_FLAG_OPER_SPY;
  }
}
break;
case 218:
#line 1561 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_HIDDEN_OPER;
    else yy_aconf->port |= OPER_FLAG_HIDDEN_OPER;
  }
}
break;
case 219:
#line 1568 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_REMOTEBAN;
    else yy_aconf->port |= OPER_FLAG_REMOTEBAN;
  }
}
break;
case 220:
#line 1575 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) ClearConfEncrypted(yy_aconf);
    else SetConfEncrypted(yy_aconf);
  }
}
break;
case 221:
#line 1582 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->port &= ~OPER_FLAG_SNOOP;
    else yy_aconf->port |= OPER_FLAG_SNOOP;
  }
}
break;
case 222:
#line 1594 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(WEBIRC_TYPE);
    yy_aconf = map_to_conf(yy_conf);
  }
}
break;
case 223:
#line 1601 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = NULL;
    yy_aconf = NULL;
  }
}
break;
case 229:
#line 1613 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_aconf->host);
    DupString(yy_aconf->host, yylval.string);
  }
}
break;
case 230:
#line 1622 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yy_aconf->passwd != NULL)
      memset(yy_aconf->passwd, 0, strlen(yy_aconf->passwd));

    MyFree(yy_aconf->passwd);
    DupString(yy_aconf->passwd, yylval.string);
  }
}
break;
case 231:
#line 1637 "ircd_parser.y"
{
  if (ypass == 1)
  {
    yy_conf = make_conf_item(CLASS_TYPE);
    yy_class = map_to_conf(yy_conf);
  }
}
break;
case 232:
#line 1644 "ircd_parser.y"
{
  if (ypass == 1)
  {
    struct ConfItem *cconf = NULL;
    struct ClassItem *class = NULL;

    if (yy_class_name == NULL)
      delete_conf_item(yy_conf);
    else
    {
      cconf = find_exact_name_conf(CLASS_TYPE, yy_class_name, NULL, NULL);

      if (cconf != NULL)		/* The class existed already */
      {
        int user_count = 0;

        rebuild_cidr_class(cconf, yy_class);

        class = map_to_conf(cconf);

        user_count = class->curr_user_count;
        memcpy(class, yy_class, sizeof(*class));
        class->curr_user_count = user_count;
        class->active = 1;

        delete_conf_item(yy_conf);

        MyFree(cconf->name);            /* Allows case change of class name */
        cconf->name = yy_class_name;
      }
      else	/* Brand new class */
      {
        MyFree(yy_conf->name);          /* just in case it was allocated */
        yy_conf->name = yy_class_name;
        yy_class->active = 1;
      }
    }

    yy_class_name = NULL;
  }
}
break;
case 251:
#line 1704 "ircd_parser.y"
{
  if (ypass == 1)
  {
    MyFree(yy_class_name);
    DupString(yy_class_name, yylval.string);
  }
}
break;
case 252:
#line 1713 "ircd_parser.y"
{
  if (ypass == 1)
  {
    MyFree(yy_class_name);
    DupString(yy_class_name, yylval.string);
  }
}
break;
case 253:
#line 1722 "ircd_parser.y"
{
  if (ypass == 1)
    PingFreq(yy_class) = yyvsp[-1].number;
}
break;
case 254:
#line 1728 "ircd_parser.y"
{
  if (ypass == 1)
    PingWarning(yy_class) = yyvsp[-1].number;
}
break;
case 255:
#line 1734 "ircd_parser.y"
{
  if (ypass == 1)
    MaxPerIp(yy_class) = yyvsp[-1].number;
}
break;
case 256:
#line 1740 "ircd_parser.y"
{
  if (ypass == 1)
    ConFreq(yy_class) = yyvsp[-1].number;
}
break;
case 257:
#line 1746 "ircd_parser.y"
{
  if (ypass == 1)
    MaxTotal(yy_class) = yyvsp[-1].number;
}
break;
case 258:
#line 1752 "ircd_parser.y"
{
  if (ypass == 1)
    MaxGlobal(yy_class) = yyvsp[-1].number;
}
break;
case 259:
#line 1758 "ircd_parser.y"
{
  if (ypass == 1)
    MaxLocal(yy_class) = yyvsp[-1].number;
}
break;
case 260:
#line 1764 "ircd_parser.y"
{
  if (ypass == 1)
    MaxIdent(yy_class) = yyvsp[-1].number;
}
break;
case 261:
#line 1770 "ircd_parser.y"
{
  if (ypass == 1)
    MaxSendq(yy_class) = yyvsp[-1].number;
}
break;
case 262:
#line 1776 "ircd_parser.y"
{
  if (ypass == 1)
    CidrBitlenIPV4(yy_class) = yyvsp[-1].number;
}
break;
case 263:
#line 1782 "ircd_parser.y"
{
  if (ypass == 1)
    CidrBitlenIPV6(yy_class) = yyvsp[-1].number;
}
break;
case 264:
#line 1788 "ircd_parser.y"
{
  if (ypass == 1)
    NumberPerCidr(yy_class) = yyvsp[-1].number;
}
break;
case 265:
#line 1797 "ircd_parser.y"
{
  if (ypass == 2)
  {
    listener_address = NULL;
    listener_flags = 0;
  }
}
break;
case 266:
#line 1804 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(listener_address);
    listener_address = NULL;
  }
}
break;
case 267:
#line 1813 "ircd_parser.y"
{
  listener_flags = 0;
}
break;
case 271:
#line 1819 "ircd_parser.y"
{
  if (ypass == 2)
    listener_flags |= LISTENER_SSL;
}
break;
case 272:
#line 1823 "ircd_parser.y"
{
  if (ypass == 2)
    listener_flags |= LISTENER_HIDDEN;
}
break;
case 280:
#line 1831 "ircd_parser.y"
{ listener_flags = 0; }
break;
case 284:
#line 1836 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if ((listener_flags & LISTENER_SSL))
#ifdef HAVE_LIBCRYPTO
      if (!ServerInfo.ctx)
#endif
      {
        yyerror("SSL not available - port closed");
	break;
      }
    add_listener(yyvsp[0].number, listener_address, listener_flags);
  }
}
break;
case 285:
#line 1850 "ircd_parser.y"
{
  if (ypass == 2)
  {
    int i;

    if ((listener_flags & LISTENER_SSL))
#ifdef HAVE_LIBCRYPTO
      if (!ServerInfo.ctx)
#endif
      {
        yyerror("SSL not available - port closed");
	break;
      }

    for (i = yyvsp[-2].number; i <= yyvsp[0].number; ++i)
      add_listener(i, listener_address, listener_flags);
  }
}
break;
case 286:
#line 1870 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(listener_address);
    DupString(listener_address, yylval.string);
  }
}
break;
case 287:
#line 1879 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(listener_address);
    DupString(listener_address, yylval.string);
  }
}
break;
case 288:
#line 1891 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(CLIENT_TYPE);
    yy_aconf = map_to_conf(yy_conf);
  }
  else
  {
    MyFree(class_name);
    class_name = NULL;
  }
}
break;
case 289:
#line 1903 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_tmp = NULL;
    dlink_node *ptr = NULL, *next_ptr = NULL;

    if (yy_aconf->user && yy_aconf->host)
    {
      conf_add_class_to_conf(yy_conf, class_name);
      add_conf_by_address(CONF_CLIENT, yy_aconf);
    }
    else
      delete_conf_item(yy_conf);

    /* copy over settings from first struct */
    DLINK_FOREACH_SAFE(ptr, next_ptr, col_conf_list.head)
    {
      struct AccessItem *new_aconf;
      struct ConfItem *new_conf;

      new_conf = make_conf_item(CLIENT_TYPE);
      new_aconf = map_to_conf(new_conf);

      yy_tmp = ptr->data;

      assert(yy_tmp->user && yy_tmp->host);

      if (yy_aconf->passwd != NULL)
        DupString(new_aconf->passwd, yy_aconf->passwd);
      if (yy_conf->name != NULL)
        DupString(new_conf->name, yy_conf->name);
      if (yy_aconf->passwd != NULL)
        DupString(new_aconf->passwd, yy_aconf->passwd);

      new_aconf->flags = yy_aconf->flags;
      new_aconf->port  = yy_aconf->port;

      DupString(new_aconf->user, yy_tmp->user);
      collapse(new_aconf->user);

      DupString(new_aconf->host, yy_tmp->host);
      collapse(new_aconf->host);

      conf_add_class_to_conf(new_conf, class_name);
      add_conf_by_address(CONF_CLIENT, new_aconf);
      dlinkDelete(&yy_tmp->node, &col_conf_list);
      free_collect_item(yy_tmp);
    }

    MyFree(class_name);
    class_name = NULL;
    yy_conf = NULL;
    yy_aconf = NULL;
  }
}
break;
case 309:
#line 1968 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_tmp = NULL;
    struct split_nuh_item nuh;

    nuh.nuhmask  = yylval.string;
    nuh.nickptr  = NULL;
    nuh.userptr  = userbuf;
    nuh.hostptr  = hostbuf;

    nuh.nicksize = 0;
    nuh.usersize = sizeof(userbuf);
    nuh.hostsize = sizeof(hostbuf);

    split_nuh(&nuh);

    if (yy_aconf->user == NULL)
    {
      DupString(yy_aconf->user, userbuf);
      DupString(yy_aconf->host, hostbuf);
    }
    else
    {
      yy_tmp = MyMalloc(sizeof(struct CollectItem));

      DupString(yy_tmp->user, userbuf);
      DupString(yy_tmp->host, hostbuf);

      dlinkAdd(yy_tmp, &yy_tmp->node, &col_conf_list);
    }
  }
}
break;
case 310:
#line 2005 "ircd_parser.y"
{
  if (ypass == 2)
  {
    /* be paranoid */
    if (yy_aconf->passwd != NULL)
      memset(yy_aconf->passwd, 0, strlen(yy_aconf->passwd));

    MyFree(yy_aconf->passwd);
    DupString(yy_aconf->passwd, yylval.string);
  }
}
break;
case 311:
#line 2018 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_SPOOF_NOTICE;
    else
      yy_aconf->flags &= ~CONF_FLAGS_SPOOF_NOTICE;
  }
}
break;
case 312:
#line 2029 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(class_name);
    DupString(class_name, yylval.string);
  }
}
break;
case 313:
#line 2038 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      SetConfEncrypted(yy_aconf);
    else
      ClearConfEncrypted(yy_aconf);
  }
}
break;
case 314:
#line 2049 "ircd_parser.y"
{
}
break;
case 318:
#line 2053 "ircd_parser.y"
{ not_atom = 1; }
break;
case 320:
#line 2054 "ircd_parser.y"
{ not_atom = 0; }
break;
case 322:
#line 2057 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_SPOOF_NOTICE;
    else yy_aconf->flags |= CONF_FLAGS_SPOOF_NOTICE;
  }

}
break;
case 323:
#line 2065 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_NOLIMIT;
    else yy_aconf->flags |= CONF_FLAGS_NOLIMIT;
  }
}
break;
case 324:
#line 2072 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_EXEMPTKLINE;
    else yy_aconf->flags |= CONF_FLAGS_EXEMPTKLINE;
  } 
}
break;
case 325:
#line 2079 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_NEED_IDENTD;
    else yy_aconf->flags |= CONF_FLAGS_NEED_IDENTD;
  }
}
break;
case 326:
#line 2086 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_CAN_FLOOD;
    else yy_aconf->flags |= CONF_FLAGS_CAN_FLOOD;
  }
}
break;
case 327:
#line 2093 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_IDLE_LINED;
    else yy_aconf->flags |= CONF_FLAGS_IDLE_LINED;
  }
}
break;
case 328:
#line 2100 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_NO_TILDE;
    else yy_aconf->flags |= CONF_FLAGS_NO_TILDE;
  } 
}
break;
case 329:
#line 2107 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_EXEMPTGLINE;
    else yy_aconf->flags |= CONF_FLAGS_EXEMPTGLINE;
  } 
}
break;
case 330:
#line 2114 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_EXEMPTRESV;
    else yy_aconf->flags |= CONF_FLAGS_EXEMPTRESV;
  }
}
break;
case 331:
#line 2121 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom) yy_aconf->flags &= ~CONF_FLAGS_NEED_PASSWORD;
    else yy_aconf->flags |= CONF_FLAGS_NEED_PASSWORD;
  }
}
break;
case 332:
#line 2130 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_EXEMPTKLINE;
    else
      yy_aconf->flags &= ~CONF_FLAGS_EXEMPTKLINE;
  }
}
break;
case 333:
#line 2141 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_NEED_IDENTD;
    else
      yy_aconf->flags &= ~CONF_FLAGS_NEED_IDENTD;
  }
}
break;
case 334:
#line 2152 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_NOLIMIT;
    else
      yy_aconf->flags &= ~CONF_FLAGS_NOLIMIT;
  }
}
break;
case 335:
#line 2163 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_CAN_FLOOD;
    else
      yy_aconf->flags &= ~CONF_FLAGS_CAN_FLOOD;
  }
}
break;
case 336:
#line 2174 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_NO_TILDE;
    else
      yy_aconf->flags &= ~CONF_FLAGS_NO_TILDE;
  }
}
break;
case 337:
#line 2185 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_EXEMPTGLINE;
    else
      yy_aconf->flags &= ~CONF_FLAGS_EXEMPTGLINE;
  }
}
break;
case 338:
#line 2197 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_conf->name);

    if (strlen(yylval.string) < HOSTLEN)
    {    
      DupString(yy_conf->name, yylval.string);
      yy_aconf->flags |= CONF_FLAGS_SPOOF_IP;
    }
    else
    {
      ilog(L_ERROR, "Spoofs must be less than %d..ignoring it", HOSTLEN);
      yy_conf->name = NULL;
    }
  }
}
break;
case 339:
#line 2216 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_aconf->flags |= CONF_FLAGS_REDIR;
    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 340:
#line 2226 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_aconf->flags |= CONF_FLAGS_REDIR;
    yy_aconf->port = yyvsp[-1].number;
  }
}
break;
case 341:
#line 2235 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_NEED_PASSWORD;
    else
      yy_aconf->flags &= ~CONF_FLAGS_NEED_PASSWORD;
  }
}
break;
case 342:
#line 2250 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(resv_reason);
    resv_reason = NULL;
  }
}
break;
case 343:
#line 2257 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(resv_reason);
    resv_reason = NULL;
  }
}
break;
case 350:
#line 2269 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(resv_reason);
    DupString(resv_reason, yylval.string);
  }
}
break;
case 351:
#line 2278 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (IsChanPrefix(*yylval.string))
    {
      char def_reason[] = "No reason";

      create_channel_resv(yylval.string, resv_reason != NULL ? resv_reason : def_reason, 1);
    }
  }
  /* ignore it for now.. but we really should make a warning if
   * its an erroneous name --fl_ */
}
break;
case 352:
#line 2293 "ircd_parser.y"
{
  if (ypass == 2)
  {
    char def_reason[] = "No reason";

    create_nick_resv(yylval.string, resv_reason != NULL ? resv_reason : def_reason, 1);
  }
}
break;
case 353:
#line 2306 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(ULINE_TYPE);
    yy_match_item = map_to_conf(yy_conf);
    yy_match_item->action = SHARED_ALL;
  }
}
break;
case 354:
#line 2314 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = NULL;
  }
}
break;
case 361:
#line 2325 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 362:
#line 2334 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct split_nuh_item nuh;

    nuh.nuhmask  = yylval.string;
    nuh.nickptr  = NULL;
    nuh.userptr  = userbuf;
    nuh.hostptr  = hostbuf;

    nuh.nicksize = 0;
    nuh.usersize = sizeof(userbuf);
    nuh.hostsize = sizeof(hostbuf);

    split_nuh(&nuh);

    DupString(yy_match_item->user, userbuf);
    DupString(yy_match_item->host, hostbuf);
  }
}
break;
case 363:
#line 2356 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action = 0;
}
break;
case 367:
#line 2363 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_KLINE;
}
break;
case 368:
#line 2367 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_TKLINE;
}
break;
case 369:
#line 2371 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_UNKLINE;
}
break;
case 370:
#line 2375 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_XLINE;
}
break;
case 371:
#line 2379 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_TXLINE;
}
break;
case 372:
#line 2383 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_UNXLINE;
}
break;
case 373:
#line 2387 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_RESV;
}
break;
case 374:
#line 2391 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_TRESV;
}
break;
case 375:
#line 2395 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_UNRESV;
}
break;
case 376:
#line 2399 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_LOCOPS;
}
break;
case 377:
#line 2403 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action |= SHARED_USERMODE;
}
break;
case 378:
#line 2407 "ircd_parser.y"
{
  if (ypass == 2)
    yy_match_item->action = SHARED_ALL;
}
break;
case 379:
#line 2416 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(CLUSTER_TYPE);
    yy_conf->flags = SHARED_ALL;
  }
}
break;
case 380:
#line 2423 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yy_conf->name == NULL)
      DupString(yy_conf->name, "*");
    yy_conf = NULL;
  }
}
break;
case 386:
#line 2436 "ircd_parser.y"
{
  if (ypass == 2)
    DupString(yy_conf->name, yylval.string);
}
break;
case 387:
#line 2442 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags = 0;
}
break;
case 391:
#line 2449 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_KLINE;
}
break;
case 392:
#line 2453 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_TKLINE;
}
break;
case 393:
#line 2457 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_UNKLINE;
}
break;
case 394:
#line 2461 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_XLINE;
}
break;
case 395:
#line 2465 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_TXLINE;
}
break;
case 396:
#line 2469 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_UNXLINE;
}
break;
case 397:
#line 2473 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_RESV;
}
break;
case 398:
#line 2477 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_TRESV;
}
break;
case 399:
#line 2481 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_UNRESV;
}
break;
case 400:
#line 2485 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags |= SHARED_LOCOPS;
}
break;
case 401:
#line 2489 "ircd_parser.y"
{
  if (ypass == 2)
    yy_conf->flags = SHARED_ALL;
}
break;
case 402:
#line 2498 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(SERVER_TYPE);
    yy_aconf = (struct AccessItem *)map_to_conf(yy_conf);
    yy_aconf->passwd = NULL;
    /* defaults */
    yy_aconf->port = PORTNUM;

    if (ConfigFileEntry.burst_away)
      yy_aconf->flags = CONF_FLAGS_BURST_AWAY;
  }
  else
  {
    MyFree(class_name);
    class_name = NULL;
  }
}
break;
case 403:
#line 2516 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_hconf=NULL;
    struct CollectItem *yy_lconf=NULL;
    dlink_node *ptr;
    dlink_node *next_ptr;
#ifdef HAVE_LIBCRYPTO
    if (yy_aconf->host &&
	((yy_aconf->passwd && yy_aconf->spasswd) ||
	 (yy_aconf->rsa_public_key && IsConfCryptLink(yy_aconf))))
#else /* !HAVE_LIBCRYPTO */
      if (yy_aconf->host && !IsConfCryptLink(yy_aconf) && 
	  yy_aconf->passwd && yy_aconf->spasswd)
#endif /* !HAVE_LIBCRYPTO */
	{
	  if (conf_add_server(yy_conf, class_name) == -1)
	  {
	    delete_conf_item(yy_conf);
	    yy_conf = NULL;
	    yy_aconf = NULL;
	  }
	}
	else
	{
	  /* Even if yy_conf ->name is NULL
	   * should still unhook any hub/leaf confs still pending
	   */
	  unhook_hub_leaf_confs();

	  if (yy_conf->name != NULL)
	  {
#ifndef HAVE_LIBCRYPTO
	    if (IsConfCryptLink(yy_aconf))
	      yyerror("Ignoring connect block -- no OpenSSL support");
#else
	    if (IsConfCryptLink(yy_aconf) && !yy_aconf->rsa_public_key)
	      yyerror("Ignoring connect block -- missing key");
#endif
	    if (yy_aconf->host == NULL)
	      yyerror("Ignoring connect block -- missing host");
	    else if (!IsConfCryptLink(yy_aconf) && 
		    (!yy_aconf->passwd || !yy_aconf->spasswd))
              yyerror("Ignoring connect block -- missing password");
	  }


          /* XXX
           * This fixes a try_connections() core (caused by invalid class_ptr
           * pointers) reported by metalrock. That's an ugly fix, but there
           * is currently no better way. The entire config subsystem needs an
           * rewrite ASAP. make_conf_item() shouldn't really add things onto
           * a doubly linked list immediately without any sanity checks!  -Michael
           */
          delete_conf_item(yy_conf);

          yy_aconf = NULL;
	  yy_conf = NULL;
	}

      /*
       * yy_conf is still pointing at the server that is having
       * a connect block built for it. This means, y_aconf->name 
       * points to the actual irc name this server will be known as.
       * Now this new server has a set or even just one hub_mask (or leaf_mask)
       * given in the link list at yy_hconf. Fill in the HUB confs
       * from this link list now.
       */        
      DLINK_FOREACH_SAFE(ptr, next_ptr, hub_conf_list.head)
      {
	struct ConfItem *new_hub_conf;
	struct MatchItem *match_item;

	yy_hconf = ptr->data;

	/* yy_conf == NULL is a fatal error for this connect block! */
	if ((yy_conf != NULL) && (yy_conf->name != NULL))
	{
	  new_hub_conf = make_conf_item(HUB_TYPE);
	  match_item = (struct MatchItem *)map_to_conf(new_hub_conf);
	  DupString(new_hub_conf->name, yy_conf->name);
	  if (yy_hconf->user != NULL)
	    DupString(match_item->user, yy_hconf->user);
	  else
	    DupString(match_item->user, "*");
	  if (yy_hconf->host != NULL)
	    DupString(match_item->host, yy_hconf->host);
	  else
	    DupString(match_item->host, "*");
	}
	dlinkDelete(&yy_hconf->node, &hub_conf_list);
	free_collect_item(yy_hconf);
      }

      /* Ditto for the LEAF confs */

      DLINK_FOREACH_SAFE(ptr, next_ptr, leaf_conf_list.head)
      {
	struct ConfItem *new_leaf_conf;
	struct MatchItem *match_item;

	yy_lconf = ptr->data;

	if ((yy_conf != NULL) && (yy_conf->name != NULL))
	{
	  new_leaf_conf = make_conf_item(LEAF_TYPE);
	  match_item = (struct MatchItem *)map_to_conf(new_leaf_conf);
	  DupString(new_leaf_conf->name, yy_conf->name);
	  if (yy_lconf->user != NULL)
	    DupString(match_item->user, yy_lconf->user);
	  else
	    DupString(match_item->user, "*");
	  if (yy_lconf->host != NULL)
	    DupString(match_item->host, yy_lconf->host);
	  else
	    DupString(match_item->host, "*");
	}
	dlinkDelete(&yy_lconf->node, &leaf_conf_list);
	free_collect_item(yy_lconf);
      }
      MyFree(class_name);
      class_name = NULL;
      yy_conf = NULL;
      yy_aconf = NULL;
  }
}
break;
case 428:
#line 2655 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yy_conf->name != NULL)
      yyerror("Multiple connect name entry");

    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 429:
#line 2667 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yy_conf->name != NULL)
      yyerror("Multiple connect name entry");

    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 430:
#line 2679 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_aconf->host);
    DupString(yy_aconf->host, yylval.string);
  }
}
break;
case 431:
#line 2688 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct addrinfo hints, *res;

    memset(&hints, 0, sizeof(hints));

    hints.ai_family   = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags    = AI_PASSIVE | AI_NUMERICHOST;

    if (irc_getaddrinfo(yylval.string, NULL, &hints, &res))
      ilog(L_ERROR, "Invalid netmask for server vhost(%s)", yylval.string);
    else
    {
      assert(res != NULL);

      memcpy(&yy_aconf->my_ipnum, res->ai_addr, res->ai_addrlen);
      yy_aconf->my_ipnum.ss.ss_family = res->ai_family;
      yy_aconf->my_ipnum.ss_len = res->ai_addrlen;
      irc_freeaddrinfo(res);
    }
  }
}
break;
case 432:
#line 2714 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yyvsp[-1].string[0] == ':')
      yyerror("Server passwords cannot begin with a colon");
    else if (strchr(yyvsp[-1].string, ' ') != NULL)
      yyerror("Server passwords cannot contain spaces");
    else {
      if (yy_aconf->spasswd != NULL)
        memset(yy_aconf->spasswd, 0, strlen(yy_aconf->spasswd));

      MyFree(yy_aconf->spasswd);
      DupString(yy_aconf->spasswd, yylval.string);
    }
  }
}
break;
case 433:
#line 2732 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yyvsp[-1].string[0] == ':')
      yyerror("Server passwords cannot begin with a colon");
    else if (strchr(yyvsp[-1].string, ' ') != NULL)
      yyerror("Server passwords cannot contain spaces");
    else {
      if (yy_aconf->passwd != NULL)
        memset(yy_aconf->passwd, 0, strlen(yy_aconf->passwd));

      MyFree(yy_aconf->passwd);
      DupString(yy_aconf->passwd, yylval.string);
    }
  }
}
break;
case 434:
#line 2750 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->port = yyvsp[-1].number;
}
break;
case 435:
#line 2756 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->aftype = AF_INET;
}
break;
case 436:
#line 2760 "ircd_parser.y"
{
#ifdef IPV6
  if (ypass == 2)
    yy_aconf->aftype = AF_INET6;
#endif
}
break;
case 437:
#line 2768 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_aconf->fakename);
    DupString(yy_aconf->fakename, yylval.string);
  }
}
break;
case 438:
#line 2777 "ircd_parser.y"
{
}
break;
case 442:
#line 2781 "ircd_parser.y"
{ not_atom = 1; }
break;
case 444:
#line 2782 "ircd_parser.y"
{ not_atom = 0; }
break;
case 446:
#line 2785 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom)ClearConfLazyLink(yy_aconf);
    else SetConfLazyLink(yy_aconf);
  }
}
break;
case 447:
#line 2792 "ircd_parser.y"
{
  if (ypass == 2)
#ifndef HAVE_LIBZ
    yyerror("Ignoring flags = compressed; -- no zlib support");
#else
 {
   if (not_atom)ClearConfCompressed(yy_aconf);
   else SetConfCompressed(yy_aconf);
 }
#endif
}
break;
case 448:
#line 2803 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom)ClearConfCryptLink(yy_aconf);
    else SetConfCryptLink(yy_aconf);
  }
}
break;
case 449:
#line 2810 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom)ClearConfAllowAutoConn(yy_aconf);
    else SetConfAllowAutoConn(yy_aconf);
  }
}
break;
case 450:
#line 2817 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom)ClearConfAwayBurst(yy_aconf);
    else SetConfAwayBurst(yy_aconf);
  }
}
break;
case 451:
#line 2824 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (not_atom)ClearConfTopicBurst(yy_aconf);
    else SetConfTopicBurst(yy_aconf);
  }
}
break;
case 452:
#line 2834 "ircd_parser.y"
{
#ifdef HAVE_LIBCRYPTO
  if (ypass == 2)
  {
    BIO *file;

    if (yy_aconf->rsa_public_key != NULL)
    {
      RSA_free(yy_aconf->rsa_public_key);
      yy_aconf->rsa_public_key = NULL;
    }

    if (yy_aconf->rsa_public_key_file != NULL)
    {
      MyFree(yy_aconf->rsa_public_key_file);
      yy_aconf->rsa_public_key_file = NULL;
    }

    DupString(yy_aconf->rsa_public_key_file, yylval.string);

    if ((file = BIO_new_file(yylval.string, "r")) == NULL)
    {
      yyerror("Ignoring rsa_public_key_file -- file doesn't exist");
      break;
    }

    yy_aconf->rsa_public_key = (RSA *)PEM_read_bio_RSA_PUBKEY(file, NULL, 0, NULL);

    if (yy_aconf->rsa_public_key == NULL)
    {
      yyerror("Ignoring rsa_public_key_file -- Key invalid; check key syntax.");
      break;
    }
      
    BIO_set_close(file, BIO_CLOSE);
    BIO_free(file);
  }
#endif /* HAVE_LIBCRYPTO */
}
break;
case 453:
#line 2875 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_ENCRYPTED;
    else
      yy_aconf->flags &= ~CONF_FLAGS_ENCRYPTED;
  }
}
break;
case 454:
#line 2886 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_CRYPTLINK;
    else
      yy_aconf->flags &= ~CONF_FLAGS_CRYPTLINK;
  }
}
break;
case 455:
#line 2897 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
#ifndef HAVE_LIBZ
      yyerror("Ignoring compressed=yes; -- no zlib support");
#else
      yy_aconf->flags |= CONF_FLAGS_COMPRESSED;
#endif
    else
      yy_aconf->flags &= ~CONF_FLAGS_COMPRESSED;
  }
}
break;
case 456:
#line 2912 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      yy_aconf->flags |= CONF_FLAGS_ALLOW_AUTO_CONN;
    else
      yy_aconf->flags &= ~CONF_FLAGS_ALLOW_AUTO_CONN;
  }
}
break;
case 457:
#line 2923 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.number)
      SetConfTopicBurst(yy_aconf);
    else
      ClearConfTopicBurst(yy_aconf);
  }
}
break;
case 458:
#line 2934 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_tmp;

    yy_tmp = (struct CollectItem *)MyMalloc(sizeof(struct CollectItem));
    DupString(yy_tmp->host, yylval.string);
    DupString(yy_tmp->user, "*");
    dlinkAdd(yy_tmp, &yy_tmp->node, &hub_conf_list);
  }
}
break;
case 459:
#line 2947 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_tmp;

    yy_tmp = (struct CollectItem *)MyMalloc(sizeof(struct CollectItem));
    DupString(yy_tmp->host, yylval.string);
    DupString(yy_tmp->user, "*");
    dlinkAdd(yy_tmp, &yy_tmp->node, &leaf_conf_list);
  }
}
break;
case 460:
#line 2960 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(class_name);
    DupString(class_name, yylval.string);
  }
}
break;
case 461:
#line 2969 "ircd_parser.y"
{
#ifdef HAVE_LIBCRYPTO
  if (ypass == 2)
  {
    struct EncCapability *ecap;
    const char *cipher_name;
    int found = 0;

    yy_aconf->cipher_preference = NULL;
    cipher_name = yylval.string;

    for (ecap = CipherTable; ecap->name; ecap++)
    {
      if ((irccmp(ecap->name, cipher_name) == 0) &&
          (ecap->cap & CAP_ENC_MASK))
      {
        yy_aconf->cipher_preference = ecap;
        found = 1;
        break;
      }
    }

    if (!found)
      yyerror("Invalid cipher");
  }
#else
  if (ypass == 2)
    yyerror("Ignoring cipher_preference -- no OpenSSL support");
#endif
}
break;
case 462:
#line 3004 "ircd_parser.y"
{
  if (ypass == 2)
  {
    userbuf[0] = hostbuf[0] = reasonbuf[0] = '\0';
    regex_ban = 0;
  }
}
break;
case 463:
#line 3011 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (userbuf[0] && hostbuf[0])
    {
      if (regex_ban)
      {
        pcre *exp_user = NULL;
        pcre *exp_host = NULL;
        const char *errptr = NULL;

        if (!(exp_user = ircd_pcre_compile(userbuf, &errptr)) ||
            !(exp_host = ircd_pcre_compile(hostbuf, &errptr)))
        {
          ilog(L_ERROR, "Failed to add regular expression based K-Line: %s",
               errptr);
          break;
        }

        yy_conf = make_conf_item(RKLINE_TYPE);
        yy_aconf = map_to_conf(yy_conf);

        yy_aconf->regexuser = exp_user;
        yy_aconf->regexhost = exp_host;

        DupString(yy_aconf->user, userbuf);
        DupString(yy_aconf->host, hostbuf);

        if (reasonbuf[0])
          DupString(yy_aconf->reason, reasonbuf);
        else
          DupString(yy_aconf->reason, "No reason");
      }
      else
      {
        yy_conf = make_conf_item(KLINE_TYPE);
        yy_aconf = map_to_conf(yy_conf);

        DupString(yy_aconf->user, userbuf);
        DupString(yy_aconf->host, hostbuf);

        if (reasonbuf[0])
          DupString(yy_aconf->reason, reasonbuf);
        else
          DupString(yy_aconf->reason, "No reason");
        add_conf_by_address(CONF_KILL, yy_aconf);
      }
    }
    else
      delete_conf_item(yy_conf);

    yy_conf = NULL;
    yy_aconf = NULL;
  }
}
break;
case 464:
#line 3068 "ircd_parser.y"
{
}
break;
case 468:
#line 3073 "ircd_parser.y"
{
  if (ypass == 2)
    regex_ban = 1;
}
break;
case 475:
#line 3082 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct split_nuh_item nuh;

    nuh.nuhmask  = yylval.string;
    nuh.nickptr  = NULL;
    nuh.userptr  = userbuf;
    nuh.hostptr  = hostbuf;

    nuh.nicksize = 0;
    nuh.usersize = sizeof(userbuf);
    nuh.hostsize = sizeof(hostbuf);

    split_nuh(&nuh);
  }
}
break;
case 476:
#line 3101 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(reasonbuf, yylval.string, sizeof(reasonbuf));
}
break;
case 477:
#line 3110 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(DLINE_TYPE);
    yy_aconf = map_to_conf(yy_conf);
    /* default reason */
    DupString(yy_aconf->reason, "No reason");
  }
}
break;
case 478:
#line 3119 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yy_aconf->host && parse_netmask(yy_aconf->host, NULL, NULL) != HM_HOST)
      add_conf_by_address(CONF_DLINE, yy_aconf);
    else
      delete_conf_item(yy_conf);
    yy_conf = NULL;
    yy_aconf = NULL;
  }
}
break;
case 484:
#line 3135 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_aconf->host);
    DupString(yy_aconf->host, yylval.string);
  }
}
break;
case 485:
#line 3144 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(yy_aconf->reason);
    DupString(yy_aconf->reason, yylval.string);
  }
}
break;
case 491:
#line 3161 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (yylval.string[0] && parse_netmask(yylval.string, NULL, NULL) != HM_HOST)
    {
      yy_conf = make_conf_item(EXEMPTDLINE_TYPE);
      yy_aconf = map_to_conf(yy_conf);
      DupString(yy_aconf->host, yylval.string);

      add_conf_by_address(CONF_EXEMPTDLINE, yy_aconf);

      yy_conf = NULL;
      yy_aconf = NULL;
    }
  }
}
break;
case 492:
#line 3182 "ircd_parser.y"
{
  if (ypass == 2)
  {
    regex_ban = 0;
    reasonbuf[0] = gecos_name[0] = '\0';
  }
}
break;
case 493:
#line 3189 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (gecos_name[0])
    {
      if (regex_ban)
      {
        pcre *exp_p = NULL;
        const char *errptr = NULL;

        if (!(exp_p = ircd_pcre_compile(gecos_name, &errptr)))
        {
          ilog(L_ERROR, "Failed to add regular expression based X-Line: %s",
               errptr);
          break;
        }

        yy_conf = make_conf_item(RXLINE_TYPE);
        yy_conf->regexpname = exp_p;
      }
      else
        yy_conf = make_conf_item(XLINE_TYPE);

      yy_match_item = map_to_conf(yy_conf);
      DupString(yy_conf->name, gecos_name);

      if (reasonbuf[0])
        DupString(yy_match_item->reason, reasonbuf);
      else
        DupString(yy_match_item->reason, "No reason");
    }
  }
}
break;
case 494:
#line 3224 "ircd_parser.y"
{
}
break;
case 498:
#line 3229 "ircd_parser.y"
{
  if (ypass == 2)
    regex_ban = 1;
}
break;
case 505:
#line 3238 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(gecos_name, yylval.string, sizeof(gecos_name));
}
break;
case 506:
#line 3244 "ircd_parser.y"
{
  if (ypass == 2)
    strlcpy(reasonbuf, yylval.string, sizeof(reasonbuf));
}
break;
case 566:
#line 3289 "ircd_parser.y"
{
  ConfigFileEntry.gline_min_cidr = yyvsp[-1].number;
}
break;
case 567:
#line 3294 "ircd_parser.y"
{
  ConfigFileEntry.gline_min_cidr6 = yyvsp[-1].number;
}
break;
case 568:
#line 3299 "ircd_parser.y"
{
  ConfigFileEntry.burst_away = yylval.number;
}
break;
case 569:
#line 3304 "ircd_parser.y"
{
  ConfigFileEntry.use_whois_actually = yylval.number;
}
break;
case 570:
#line 3309 "ircd_parser.y"
{
  GlobalSetOptions.rejecttime = yylval.number;
}
break;
case 571:
#line 3314 "ircd_parser.y"
{
  ConfigFileEntry.tkline_expire_notices = yylval.number;
}
break;
case 572:
#line 3319 "ircd_parser.y"
{
  ConfigFileEntry.kill_chase_time_limit = yyvsp[-1].number;
}
break;
case 573:
#line 3324 "ircd_parser.y"
{
  ConfigFileEntry.hide_spoof_ips = yylval.number;
}
break;
case 574:
#line 3329 "ircd_parser.y"
{
  ConfigFileEntry.ignore_bogus_ts = yylval.number;
}
break;
case 575:
#line 3334 "ircd_parser.y"
{
  ConfigFileEntry.disable_remote = yylval.number;
}
break;
case 576:
#line 3339 "ircd_parser.y"
{
  ConfigFileEntry.failed_oper_notice = yylval.number;
}
break;
case 577:
#line 3344 "ircd_parser.y"
{
  ConfigFileEntry.anti_nick_flood = yylval.number;
}
break;
case 578:
#line 3349 "ircd_parser.y"
{
  ConfigFileEntry.max_nick_time = yyvsp[-1].number; 
}
break;
case 579:
#line 3354 "ircd_parser.y"
{
  ConfigFileEntry.max_nick_changes = yyvsp[-1].number;
}
break;
case 580:
#line 3359 "ircd_parser.y"
{
  ConfigFileEntry.max_accept = yyvsp[-1].number;
}
break;
case 581:
#line 3364 "ircd_parser.y"
{
  ConfigFileEntry.anti_spam_exit_message_time = yyvsp[-1].number;
}
break;
case 582:
#line 3369 "ircd_parser.y"
{
  ConfigFileEntry.ts_warn_delta = yyvsp[-1].number;
}
break;
case 583:
#line 3374 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigFileEntry.ts_max_delta = yyvsp[-1].number;
}
break;
case 584:
#line 3380 "ircd_parser.y"
{
  if ((yyvsp[-1].number > 0) && ypass == 1)
  {
    ilog(L_CRIT, "You haven't read your config file properly.");
    ilog(L_CRIT, "There is a line in the example conf that will kill your server if not removed.");
    ilog(L_CRIT, "Consider actually reading/editing the conf file, and removing this line.");
    exit(0);
  }
}
break;
case 585:
#line 3391 "ircd_parser.y"
{
  ConfigFileEntry.kline_with_reason = yylval.number;
}
break;
case 586:
#line 3396 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ConfigFileEntry.kline_reason);
    DupString(ConfigFileEntry.kline_reason, yylval.string);
  }
}
break;
case 587:
#line 3405 "ircd_parser.y"
{
  ConfigFileEntry.invisible_on_connect = yylval.number;
}
break;
case 588:
#line 3410 "ircd_parser.y"
{
  ConfigFileEntry.warn_no_nline = yylval.number;
}
break;
case 589:
#line 3415 "ircd_parser.y"
{
  ConfigFileEntry.stats_e_disabled = yylval.number;
}
break;
case 590:
#line 3420 "ircd_parser.y"
{
  ConfigFileEntry.stats_o_oper_only = yylval.number;
}
break;
case 591:
#line 3425 "ircd_parser.y"
{
  ConfigFileEntry.stats_P_oper_only = yylval.number;
}
break;
case 592:
#line 3430 "ircd_parser.y"
{
  ConfigFileEntry.stats_k_oper_only = 2 * yylval.number;
}
break;
case 593:
#line 3433 "ircd_parser.y"
{
  ConfigFileEntry.stats_k_oper_only = 1;
}
break;
case 594:
#line 3438 "ircd_parser.y"
{
  ConfigFileEntry.stats_i_oper_only = 2 * yylval.number;
}
break;
case 595:
#line 3441 "ircd_parser.y"
{
  ConfigFileEntry.stats_i_oper_only = 1;
}
break;
case 596:
#line 3446 "ircd_parser.y"
{
  ConfigFileEntry.pace_wait = yyvsp[-1].number;
}
break;
case 597:
#line 3451 "ircd_parser.y"
{
  ConfigFileEntry.caller_id_wait = yyvsp[-1].number;
}
break;
case 598:
#line 3456 "ircd_parser.y"
{
  ConfigFileEntry.opers_bypass_callerid = yylval.number;
}
break;
case 599:
#line 3461 "ircd_parser.y"
{
  ConfigFileEntry.pace_wait_simple = yyvsp[-1].number;
}
break;
case 600:
#line 3466 "ircd_parser.y"
{
  ConfigFileEntry.short_motd = yylval.number;
}
break;
case 601:
#line 3471 "ircd_parser.y"
{
  ConfigFileEntry.no_oper_flood = yylval.number;
}
break;
case 602:
#line 3476 "ircd_parser.y"
{
  ConfigFileEntry.true_no_oper_flood = yylval.number;
}
break;
case 603:
#line 3481 "ircd_parser.y"
{
  ConfigFileEntry.oper_pass_resv = yylval.number;
}
break;
case 604:
#line 3486 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if (strlen(yylval.string) > LOCALE_LENGTH-2)
      yylval.string[LOCALE_LENGTH-1] = '\0';

    set_locale(yylval.string);
  }
}
break;
case 605:
#line 3497 "ircd_parser.y"
{
  ConfigFileEntry.idletime = yyvsp[-1].number;
}
break;
case 606:
#line 3502 "ircd_parser.y"
{
  ConfigFileEntry.dots_in_ident = yyvsp[-1].number;
}
break;
case 607:
#line 3507 "ircd_parser.y"
{
  ConfigFileEntry.max_targets = yyvsp[-1].number;
}
break;
case 608:
#line 3512 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ConfigFileEntry.servlink_path);
    DupString(ConfigFileEntry.servlink_path, yylval.string);
  }
}
break;
case 609:
#line 3521 "ircd_parser.y"
{
#ifdef HAVE_LIBCRYPTO
  if (ypass == 2)
  {
    struct EncCapability *ecap;
    const char *cipher_name;
    int found = 0;

    ConfigFileEntry.default_cipher_preference = NULL;
    cipher_name = yylval.string;

    for (ecap = CipherTable; ecap->name; ecap++)
    {
      if ((irccmp(ecap->name, cipher_name) == 0) &&
          (ecap->cap & CAP_ENC_MASK))
      {
        ConfigFileEntry.default_cipher_preference = ecap;
        found = 1;
        break;
      }
    }

    if (!found)
      yyerror("Invalid cipher");
  }
#else
  if (ypass == 2)
    yyerror("Ignoring default_cipher_preference -- no OpenSSL support");
#endif
}
break;
case 610:
#line 3553 "ircd_parser.y"
{
  if (ypass == 2)
  {
    ConfigFileEntry.compression_level = yyvsp[-1].number;
#ifndef HAVE_LIBZ
    yyerror("Ignoring compression_level -- no zlib support");
#else
    if ((ConfigFileEntry.compression_level < 1) ||
        (ConfigFileEntry.compression_level > 9))
    {
      yyerror("Ignoring invalid compression_level, using default");
      ConfigFileEntry.compression_level = 0;
    }
#endif
  }
}
break;
case 611:
#line 3571 "ircd_parser.y"
{
  ConfigFileEntry.use_egd = yylval.number;
}
break;
case 612:
#line 3576 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ConfigFileEntry.egdpool_path);
    DupString(ConfigFileEntry.egdpool_path, yylval.string);
  }
}
break;
case 613:
#line 3585 "ircd_parser.y"
{
  ConfigFileEntry.ping_cookie = yylval.number;
}
break;
case 614:
#line 3590 "ircd_parser.y"
{
  ConfigFileEntry.disable_auth = yylval.number;
}
break;
case 615:
#line 3595 "ircd_parser.y"
{
  ConfigFileEntry.throttle_time = yylval.number;
}
break;
case 616:
#line 3600 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes = 0;
}
break;
case 620:
#line 3606 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_BOTS;
}
break;
case 621:
#line 3609 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_CCONN;
}
break;
case 622:
#line 3612 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_CCONN_FULL;
}
break;
case 623:
#line 3615 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_DEAF;
}
break;
case 624:
#line 3618 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_DEBUG;
}
break;
case 625:
#line 3621 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_FULL;
}
break;
case 626:
#line 3624 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_SKILL;
}
break;
case 627:
#line 3627 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_NCHANGE;
}
break;
case 628:
#line 3630 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_REJ;
}
break;
case 629:
#line 3633 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_UNAUTH;
}
break;
case 630:
#line 3636 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_SPY;
}
break;
case 631:
#line 3639 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_EXTERNAL;
}
break;
case 632:
#line 3642 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_OPERWALL;
}
break;
case 633:
#line 3645 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_SERVNOTICE;
}
break;
case 634:
#line 3648 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_INVISIBLE;
}
break;
case 635:
#line 3651 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_WALLOP;
}
break;
case 636:
#line 3654 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_SOFTCALLERID;
}
break;
case 637:
#line 3657 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_CALLERID;
}
break;
case 638:
#line 3660 "ircd_parser.y"
{
  ConfigFileEntry.oper_umodes |= UMODE_LOCOPS;
}
break;
case 639:
#line 3665 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes = 0;
}
break;
case 643:
#line 3671 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_BOTS;
}
break;
case 644:
#line 3674 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_CCONN;
}
break;
case 645:
#line 3677 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_CCONN_FULL;
}
break;
case 646:
#line 3680 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_DEAF;
}
break;
case 647:
#line 3683 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_DEBUG;
}
break;
case 648:
#line 3686 "ircd_parser.y"
{ 
  ConfigFileEntry.oper_only_umodes |= UMODE_FULL;
}
break;
case 649:
#line 3689 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_SKILL;
}
break;
case 650:
#line 3692 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_NCHANGE;
}
break;
case 651:
#line 3695 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_REJ;
}
break;
case 652:
#line 3698 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_UNAUTH;
}
break;
case 653:
#line 3701 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_SPY;
}
break;
case 654:
#line 3704 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_EXTERNAL;
}
break;
case 655:
#line 3707 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_OPERWALL;
}
break;
case 656:
#line 3710 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_SERVNOTICE;
}
break;
case 657:
#line 3713 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_INVISIBLE;
}
break;
case 658:
#line 3716 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_WALLOP;
}
break;
case 659:
#line 3719 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_SOFTCALLERID;
}
break;
case 660:
#line 3722 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_CALLERID;
}
break;
case 661:
#line 3725 "ircd_parser.y"
{
  ConfigFileEntry.oper_only_umodes |= UMODE_LOCOPS;
}
break;
case 662:
#line 3730 "ircd_parser.y"
{
  ConfigFileEntry.min_nonwildcard = yyvsp[-1].number;
}
break;
case 663:
#line 3735 "ircd_parser.y"
{
  ConfigFileEntry.min_nonwildcard_simple = yyvsp[-1].number;
}
break;
case 664:
#line 3740 "ircd_parser.y"
{
  ConfigFileEntry.default_floodcount = yyvsp[-1].number;
}
break;
case 665:
#line 3745 "ircd_parser.y"
{
  ConfigFileEntry.client_flood = yyvsp[-1].number;
}
break;
case 666:
#line 3750 "ircd_parser.y"
{
  ConfigFileEntry.dot_in_ip6_addr = yylval.number;
}
break;
case 667:
#line 3758 "ircd_parser.y"
{
  if (ypass == 2)
  {
    yy_conf = make_conf_item(GDENY_TYPE);
    yy_aconf = map_to_conf(yy_conf);
  }
}
break;
case 668:
#line 3765 "ircd_parser.y"
{
  if (ypass == 2)
  {
    /*
     * since we re-allocate yy_conf/yy_aconf after the end of action=, at the
     * end we will have one extra, so we should free it.
     */
    if (yy_conf->name == NULL || yy_aconf->user == NULL)
    {
      delete_conf_item(yy_conf);
      yy_conf = NULL;
      yy_aconf = NULL;
    }
  }
}
break;
case 678:
#line 3791 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigFileEntry.glines = yylval.number;
}
break;
case 679:
#line 3797 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigFileEntry.gline_time = yyvsp[-1].number;
}
break;
case 680:
#line 3803 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigFileEntry.gline_logging = 0;
}
break;
case 684:
#line 3809 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigFileEntry.gline_logging |= GDENY_REJECT;
}
break;
case 685:
#line 3813 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigFileEntry.gline_logging |= GDENY_BLOCK;
}
break;
case 686:
#line 3819 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct split_nuh_item nuh;

    nuh.nuhmask  = yylval.string;
    nuh.nickptr  = NULL;
    nuh.userptr  = userbuf;
    nuh.hostptr  = hostbuf;

    nuh.nicksize = 0;
    nuh.usersize = sizeof(userbuf);
    nuh.hostsize = sizeof(hostbuf);

    split_nuh(&nuh);

    if (yy_aconf->user == NULL)
    {
      DupString(yy_aconf->user, userbuf);
      DupString(yy_aconf->host, hostbuf);
    }
    else
    {
      struct CollectItem *yy_tmp = MyMalloc(sizeof(struct CollectItem));

      DupString(yy_tmp->user, userbuf);
      DupString(yy_tmp->host, hostbuf);

      dlinkAdd(yy_tmp, &yy_tmp->node, &col_conf_list);
    }
  }
}
break;
case 687:
#line 3853 "ircd_parser.y"
{
  if (ypass == 2)  
  {
    MyFree(yy_conf->name);
    DupString(yy_conf->name, yylval.string);
  }
}
break;
case 688:
#line 3862 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->flags = 0;
}
break;
case 689:
#line 3866 "ircd_parser.y"
{
  if (ypass == 2)
  {
    struct CollectItem *yy_tmp = NULL;
    dlink_node *ptr, *next_ptr;

    DLINK_FOREACH_SAFE(ptr, next_ptr, col_conf_list.head)
    {
      struct AccessItem *new_aconf;
      struct ConfItem *new_conf;

      yy_tmp = ptr->data;
      new_conf = make_conf_item(GDENY_TYPE);
      new_aconf = map_to_conf(new_conf);

      new_aconf->flags = yy_aconf->flags;

      if (yy_conf->name != NULL)
        DupString(new_conf->name, yy_conf->name);
      else
        DupString(new_conf->name, "*");
      if (yy_aconf->user != NULL)
         DupString(new_aconf->user, yy_tmp->user);
      else   
        DupString(new_aconf->user, "*");
      if (yy_aconf->host != NULL)
        DupString(new_aconf->host, yy_tmp->host);
      else
        DupString(new_aconf->host, "*");

      dlinkDelete(&yy_tmp->node, &col_conf_list);
    }

    /*
     * In case someone has fed us with more than one action= after user/name
     * which would leak memory  -Michael
     */
    if (yy_conf->name == NULL || yy_aconf->user == NULL)
      delete_conf_item(yy_conf);

    yy_conf = make_conf_item(GDENY_TYPE);
    yy_aconf = map_to_conf(yy_conf);
  }
}
break;
case 692:
#line 3913 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->flags |= GDENY_REJECT;
}
break;
case 693:
#line 3917 "ircd_parser.y"
{
  if (ypass == 2)
    yy_aconf->flags |= GDENY_BLOCK;
}
break;
case 716:
#line 3941 "ircd_parser.y"
{
  ConfigChannel.disable_fake_channels = yylval.number;
}
break;
case 717:
#line 3946 "ircd_parser.y"
{
  ConfigChannel.restrict_channels = yylval.number;
}
break;
case 718:
#line 3951 "ircd_parser.y"
{
  ConfigChannel.disable_local_channels = yylval.number;
}
break;
case 719:
#line 3956 "ircd_parser.y"
{
  ConfigChannel.use_except = yylval.number;
}
break;
case 720:
#line 3961 "ircd_parser.y"
{
  ConfigChannel.use_invex = yylval.number;
}
break;
case 721:
#line 3966 "ircd_parser.y"
{
  ConfigChannel.use_knock = yylval.number;
}
break;
case 722:
#line 3971 "ircd_parser.y"
{
  ConfigChannel.knock_delay = yyvsp[-1].number;
}
break;
case 723:
#line 3976 "ircd_parser.y"
{
  ConfigChannel.knock_delay_channel = yyvsp[-1].number;
}
break;
case 724:
#line 3981 "ircd_parser.y"
{
  ConfigChannel.max_chans_per_user = yyvsp[-1].number;
}
break;
case 725:
#line 3986 "ircd_parser.y"
{
  ConfigChannel.quiet_on_ban = yylval.number;
}
break;
case 726:
#line 3991 "ircd_parser.y"
{
  ConfigChannel.max_bans = yyvsp[-1].number;
}
break;
case 727:
#line 3996 "ircd_parser.y"
{
  ConfigChannel.default_split_user_count = yyvsp[-1].number;
}
break;
case 728:
#line 4001 "ircd_parser.y"
{
  ConfigChannel.default_split_server_count = yyvsp[-1].number;
}
break;
case 729:
#line 4006 "ircd_parser.y"
{
  ConfigChannel.no_create_on_split = yylval.number;
}
break;
case 730:
#line 4011 "ircd_parser.y"
{
  ConfigChannel.no_join_on_split = yylval.number;
}
break;
case 731:
#line 4016 "ircd_parser.y"
{
  ConfigChannel.burst_topicwho = yylval.number;
}
break;
case 732:
#line 4021 "ircd_parser.y"
{
  GlobalSetOptions.joinfloodcount = yylval.number;
}
break;
case 733:
#line 4026 "ircd_parser.y"
{
  GlobalSetOptions.joinfloodtime = yylval.number;
}
break;
case 745:
#line 4045 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigServerHide.flatten_links = yylval.number;
}
break;
case 746:
#line 4051 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigServerHide.hide_servers = yylval.number;
}
break;
case 747:
#line 4057 "ircd_parser.y"
{
  if (ypass == 2)
  {
    MyFree(ConfigServerHide.hidden_name);
    DupString(ConfigServerHide.hidden_name, yylval.string);
  }
}
break;
case 748:
#line 4066 "ircd_parser.y"
{
  if (ypass == 2)
  {
    if ((yyvsp[-1].number > 0) && ConfigServerHide.links_disabled == 1)
    {
      eventAddIsh("write_links_file", write_links_file, NULL, yyvsp[-1].number);
      ConfigServerHide.links_disabled = 0;
    }

    ConfigServerHide.links_delay = yyvsp[-1].number;
  }
}
break;
case 749:
#line 4080 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigServerHide.hidden = yylval.number;
}
break;
case 750:
#line 4086 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigServerHide.disable_hidden = yylval.number;
}
break;
case 751:
#line 4092 "ircd_parser.y"
{
  if (ypass == 2)
    ConfigServerHide.hide_server_ips = yylval.number;
}
break;
#line 6501 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
